unit FormatInfo_Form;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls;

type
  TFormatInfoForm = class(TForm)
    ListView: TListView;
    ListViewError: TListView;
    Splitter1: TSplitter;
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  end;

var
  FormatInfoForm: TFormatInfoForm;
  GflFilterString: String; // for Open-DialogBox

implementation

uses
  LibGfl;
{$R *.DFM}

procedure GetFilterString;
var
  f, i, j: Integer;
  ext, entry: String;
  info: GFL_FORMAT_INFORMATION;
  slDesc, slExt, slAll: TStringList;
begin
  slDesc:= TStringlist.Create;  // Description for each filetype
  slExt:= TStringlist.Create;  // Extension of filetype

  f:= gflGetNumberOfFormat;
  for i:= 0 to f- 1 do begin
    gflGetFormatInformationByIndex( i, info );
    ext:= gflGetDefaultFormatSuffixByIndex( i );
    if Length( ext )> 0 then begin
      if info.NumberOfExtension> 0 then begin  // Get all extensions
        ext:= '';
        for j:= 0 to info.NumberOfExtension- 1 do ext:= ext+ ';*.'+ LowerCase( info.Extension[j] );
        Delete( ext, 1, 1 );

        // Now check if (all) the extension aren't already in use by another format
        j:= slExt.IndexOf( ext );
        if j<> -1 then begin  // yes, they are. Simply add description instead of adding new entry
          slDesc[j]:= slDesc[j]+ ' / '+ info.Description;
        end else begin  // no, not yet. Add description and extensions to lists
          slDesc.Add( info.Description );
          slExt.Add( ext );
        end;
      end;
    end;
  end;

  slAll:= TStringlist.Create;  // Final list

  // Order by description
  for i:= 0 to slExt.Count- 1 do slAll.Add( Format( '|%s (%s)|%s', [slDesc[i], slExt[i], slExt[i]] ) );
  slAll.Sort;

  // This should always be the last entry
  slAll.Add( '|All files (*.*)|*.*' );

  // This should always be the first entry
  entry:= 'All supported types|';
  for i:= 0 to slExt.Count- 1 do entry:= entry+ slExt[i]+ ';';
  Delete( entry, Length( entry ), 1 );
  slAll.Insert( 0, entry );

  GflFilterString:= '';
  for i:= 0 to slAll.Count- 1 do GflFilterString:= GflFilterString+ slAll[i];

  slAll.Free;
  slDesc.Free;
  slExt.Free;
end;

procedure TFormatInfoForm.FormCreate(Sender: TObject);
var
  f, i, j: Integer;
  s, e: String;
  info: GFL_FORMAT_INFORMATION;
  item: TListItem;
const
  BOOL_STR: array[false..true] of String= ( 'No', 'Yes' );
begin
  // 2010-03-09 AmigoJack:
  //   Moved this procedure from INITIALIZATION block to here, as it has no critical
  //   operations which must be done that early! It's totally sufficient to call it
  //   here when creating this Form. This way, we can also load LibGfl dynamically
  //   instead of statically.
  GetFilterString;

  // Version Info
  Caption:= 'GFL SDK Version '+ gflGetVersion;

  // How many formats?
  f:= gflGetNumberOfFormat;
  for i:= 0 to f- 1 do begin
    if gflGetFormatInformationByIndex( i, info )= GFL_NO_ERROR then begin
      // Create new entry in ListView with description as caption
      item:= ListView.Items.Add;
      item.Caption:= info.Description;
      item.SubItems.Add( info.Name );

      // Read, Write ?
      item.SubItems.Add( BOOL_STR[info.Status and GFL_READ= GFL_READ] );
      item.SubItems.Add( BOOL_STR[info.Status and GFL_WRITE= GFL_WRITE] );

      // Extensions
      e:= '';
      for j:= 0 to info.NumberOfExtension- 1 do e:= e+ ', '+ info.Extension[j];
      Delete( e, 1, 2 );
      item.SubItems.Add( e );
    end;
  end;

  // Sort ListView
  ListView.SortType:= stText;

  // Error Strings
  for i:= 0 to 255 do begin
    s:= gflGetErrorString( i );
    if Copy( s, 1, 1 )<> '?' then begin  // if Error exists
      item:= ListViewError.Items.Add;
      item.Caption:= Format( '%.3d', [i] );
      item.SubItems.Add( s );
    end;
  end;
end;

// 2010-03-09 AmigoJack:
//   See body of "procedure TFormatInfoForm.FormCreate(Sender: TObject);"

end.
