////////////////////////////////////////////////////////////////////////////////
//  Graphics File Library
//  For Windows & Unix
//
//  GFL library Copyright (c) 1991-2010 Pierre-e Gougelet
//  All rights reserved
//
//  Commercial use is not authorized without agreement
//
//  URL:     http://www.xnview.com
//  E-Mail : webmaster@xnview.com
//
//  Version 3.11
//
////////////////////////////////////////////////////////////////////////////////
//  Found a bug? Fixed or improved something? -> Contact XnView Forum please!
//  Thanks to Ingo Neumann for first translation to Delphi
//  GPo 2008
//
////////////////////////////////////////////////////////////////////////////////
//  Delphi gflLib Version:
//  Version 3.11  ** If you update or fix anything, change this Version number **
//
//  History:
//
//   (Use this example as guide for your history entry following the formats)
//  yyyy-MM-dd: your Name
//   * Use an asterisk to mark a change, a fix, an update... anything which
//     already existed and has been modified, improved... etc.
//   + Use a plus to indicate something new which did not exist before.
//   - Use a minus/hyphen if you removed/deleted something. Also name things
//     which might still be there but have been disabled/commented out.
//   > Anything else which doesn't fit to the three characters above (like
//     generic comments also) should begin with "greater than"-sign. Also note
//     that if you need more than one line you should keep the indent. The Date
//     in the first line is yyyy=Year, MM=Month (always two digits), dd=Day
//     (always two digits).
//
//  2010-03-18: AmigoJack
//   * Reordered the whole definitions to match with the order in the header-files
//   * Rewrote all definitions of "src: PGFL_BITMAP" to "var src: GFL_BITMAP"
//     with respect to "09/03/2002 Ingo Neumann" leaving suspicious definitions
//     like "dst: PPGFL_BITMAP" intact. Nevertheless, definitions like from Ingo
//     are dangerous pitfalls - whenever a NIL won't ever occur one should use a
//     VAR-definition instead.
//   * fixed: "gflSepiaEx" which was defined as "gflSepiaExt" and therefore never
//     linked. Guess nobody ever used it until now...
//   - removed "gflBitmapSetEXIFThumbnail", "gflGetDefaultPreviewParams"
//   - removed "gflBitmapGetIsTrueColors", "gflBitmapGetIs24Bits" (in fact
//     they're only commented out because I don't think someone uses them)
//   + added types "GFL_LOAD_INFO_CB", "GFL_VIRTUAL_LOAD_CALLBACK"
//   + added functions "gflGetExtraInfosCount", "gflGetExtraInfos",
//     "gflBitmapGetEXIFValue", "gflBitmapSetEXIFValueRationalArray2" (thanks to
//     mediazg for pointing out last one)
//   * fixed translation of macro "gflGetBitmapPtr"
//   * fixed many parameter and type-definitions - too much to explain here...
//   * change: removed leading "T" form all type-definitions as following the
//     convention to begin every type with a "T" doesn't help anything at all.
//     Every type should be as exact as the source definition (from the header-
//     files). Only pointer-types should follow the convention to have the same
//     name of the type with a leading "P". That's all.
//   * Reformatting initial comments and history, also added an explanation.
//     Removed unnecessary comments and put Ingo Neumann's notes into the history.
//   > tested with Delphi 5.62 @ Windows XP Pro, Service Pack 2
//
//  2010-03-09: AmigoJack
//   + added support to dynamically load the whole library, see compiler switch
//     STATIC (default on) by also respecting Unicode/non-Unicode switches;
//     following are new:
//     > variables: "hLibGfl", "hLibGfle" plus the internal one "_bGflLibraryInit"
//     > functions: "gfl_LoadLib", "gfl_UnloadLib" plus the internal ones "_StrPas",
//       "_StrPasW", "_IntToSky", "_GetLastErrorMsg", "_gfl_GetProcAddress"
//     > constants: internal "_SKY_NUMBERS"
//
//  2008-03-04: Gisbert Pospischil (GPo)
//   + added: UNICODE compiler switches UNICODE_SUPPORT (default off) see below.
//     gflBitmapGetIsTrueColors  (local)
//     gflBitmapGetIs24Bits      (local)
//   * change: const to var bitmap in Unicode load functions gflAutoCrop and
//     gflAutoCrop2: PGFL_COLOR to TGFL_COLOR, Unicode load and save functions:
//     PFileInfo to TFileInfo and PParameters to TParameters
//   * fixed: Unicode functions, added call conversion 'stdcall'
//
//  2008-03-02: Gisbert Pospischil, change and update to v2.82
//   * change: all exif and iptc records to packed records
//   + added: declarations and functions from version 2.82
//   * fixed bugs was reported on XnView Forum.
//   > not all updates has it tested (Delphi 10 on WinXP)
//     Thanks to Pierre-e Gougelet for this greate work!
//
//  2004-06-18: Pierre-e Gougelet
//   * fixed gflRotate
//
//  2004-05-27: Pierre-e Gougelet
//   + added exif/iptc functions
//
//  2003-09-30: Pierre-e Gougelet
//
//  2002-03-09: Ingo Neumann
//   * changed all (var dst : PGFL_BITMAP) into (dst : PPGFL_BITMAP) to supress
//     compiler errors if using NIL (NULL) for dst in all LibGflE functions but
//     I am not sure if that is always correct.
//   * correct some other errors for Version 1.50, marked with  '// IN!'
//
//  2001-11-30: Germain Garand
//   * Corrected types which were inverted (swapped LongWord/LongInt)
//
//  2001-11-29: Germain Garand (germain@ebooksfrance.com)
//   + Added Kylix support (needed rename file to LibGfl.pas for case sensitivity)
//   * Corrected typo SaveBitmapFormHandle -> SaveBitmapFromHandle
//
//  2001-07-10: Ingo Neumann (ingo@upstart.de)
//   * initial creation; Tested with Delphi 5 under Windows 2000, ServicePack 2.
//     Should also work with Delphi 2.x and above under Win9x, WinME, WinNT4.
//
////////////////////////////////////////////////////////////////////////////////

unit LibGfl;

interface

uses windows;
{$ALIGN ON}  // Aligned record fields

{$IFDEF WIN32}  // KH: added for use with Win98
  {$DEFINE MSWINDOWS}
  {$DEFINE UNICODE_SUPPORT}  // fully support internationalization
{$ENDIF}


// 2010-03-09 AmigoJack:
//   New switch to chose from classic static linking to the DLLs or the new introduced
//   dynamic linking. The following is a list of disadvantages of static linking, which
//   opposite as advantages of dynamic linking:
//   - Functions and procedures are linked to the DLLs from the very start of the program,
//     there is no possibility to interfere if you need to make preparations, which leads
//     to...
//   - DLLs must be found by the system, that means they either reside in the directory of
//     the starting EXE-file, the current directory, Window's SYSTEM-directory, Window's
//     WINDOW-directory or any directory found in the PATH environment variable. If one
//     of the DLLs cannot be found, the whole won't run!
//   - DLLs are loaded always and thus consume memory, despite the fact you might never
//     need them for the current execution of the program - or at least not from the very
//     start of it.
//   Regarding the points above, dynamic linking offers more flexibility if you want to
//   implement LibGfl into your program:
//   - DLLs are not loaded initially - it is up to you to call the initialization. This
//     way you do not unnecessary consume memory. Maybe you'll never need LibGfl at all
//     for the current execution of your program.
//   - DLLs can reside in any directory you wish them. Plus their names may also differ.
//     The "GFL SDK" instructs you to copy the DLLs to Window's SYSTEM-directory - this
//     makes sure the system will always find them. But once you want to deploy your
//     program and/or want to use different versions of LibGfl and/or don't want to
//     overwrite possibly existing DLLs on the end-user's system you run into problems.
//     This is simple, but effectively BAD practice! The "GFL SDK" on top instructs you
//     to register these DLLs (with REGSVR32), which I highly discourage from! This
//     results in Windows always loading the DLLs when Windows itself starts a session
//     (that is after a user logged on). While some DLLs in the real world benefit from
//     being loaded when Windows starts most them absolutely DO NOT NEED this. Effectively
//     most program installation routines register their DLLs although those are only
//     needed when their program runs. But Windows always loads all registered DLLs, which
//     not only occupies memory but also results in a time getting longer and longer with
//     every DLL on every Windows-start! After knowing that, decide on your own if it's
//     really worth for you to register the DLLs.
//   - Because you have to manually link each function/procedure (ok, this is a downside
//     for unexperienced programmers) you can also check if each linking was successful
//     and give warnings if errors occur.
//{$DEFINE STATIC}


{$IFDEF STATIC}  // 2010-03-09 AmigoJack
const
  GflDLL  = 'libgfl311.dll';
  GfleDLL = 'libgfle311.dll';
{$ELSE}  // 2010-03-09 AmigoJack (until ENDIF)
var
  hLibGfl: THandle= 0;  // Handle to base DLL
  hLibGfle: THandle= 0;  // Handle to extended/extra DLL

  // Call this whenever you want and you're ready to link, load and initialize the whole LibGfl
  {$IFDEF UNICODE_SUPPORT}
  function gfl_LoadLib( sPathLibGfl, sPathLibGfle, sPathPlugins: WideString; var sFail: WideString ): boolean;
  function gfl_UnloadLib( var sFail: WideString ): boolean;
  {$ELSE}
  function gfl_LoadLib( sPathLibGfl, sPathLibGfle, sPathPlugins: String; var sFail: String ): boolean;
  function gfl_UnloadLib( var sFail: String ): boolean;
  {$ENDIF}
{$ENDIF}  // 2010-03-09 AmigoJack (from previous IFDEF STATIC ... ELSE)

const
  GFL_VERSION	= '311';
  GFL_VERSION_	= $0311;

  GFL_FALSE	= 0;
  GFL_TRUE	= 1;

type
  GFL_INT8= ShortInt;
  GFL_UINT8= Byte;
  GFL_INT16= SmallInt;
  GFL_UINT16= Word;
  GFL_INT32= LongInt;
  GFL_UINT32= LongWord;

  GFL_BOOL= Byte;

  PGFL_UINT8= ^GFL_UINT8; // IN!
  PGFL_UINT16= ^GFL_UINT16; //IS
  PGFL_UINT32= ^GFL_UINT32;


// ERROR
const
  GFL_NO_ERROR			= 0;
  GFL_ERROR_FILE_OPEN		= 1;
  GFL_ERROR_FILE_READ		= 2;
  GFL_ERROR_FILE_CREATE		= 3;
  GFL_ERROR_FILE_WRITE		= 4;
  GFL_ERROR_NO_MEMORY		= 5;
  GFL_ERROR_UNKNOWN_FORMAT	= 6;
  GFL_ERROR_BAD_BITMAP		= 7;
  GFL_ERROR_BAD_FORMAT_INDEX	= 10;
  GFL_ERROR_BAD_PARAMETERS	= 50;
  GFL_UNKNOWN_ERROR		= 255;
type
  GFL_ERROR= GFL_INT16;


// ORIGIN type
const
  GFL_LEFT		= $00;
  GFL_RIGHT		= $01;
  GFL_TOP		= $00;
  GFL_BOTTOM		= $10;
  GFL_TOP_LEFT		= GFL_TOP or GFL_LEFT;
  GFL_BOTTOM_LEFT	= GFL_BOTTOM or GFL_LEFT;
  GFL_TOP_RIGHT		= GFL_TOP or GFL_RIGHT;
  GFL_BOTTOM_RIGHT	= GFL_BOTTOM or GFL_RIGHT;
type
  GFL_ORIGIN= GFL_UINT16;


// COMPRESSION type
const
  GFL_NO_COMPRESSION		= 0;
  GFL_RLE			= 1;
  GFL_LZW			= 2;
  GFL_JPEG			= 3;
  GFL_ZIP			= 4;
  GFL_SGI_RLE			= 5;
  GFL_CCITT_RLE			= 6;
  GFL_CCITT_FAX3		= 7;
  GFL_CCITT_FAX3_2D		= 8;
  GFL_CCITT_FAX4		= 9;
  GFL_WAVELET			= 10;
  GFL_LZW_PREDICTOR		= 11;
  GFL_UNKNOWN_COMPRESSION	= 255;
type
  GFL_COMPRESSION= GFL_UINT16;


// BITMAP type
const
  GFL_BINARY	= $0001;
  GFL_GREY	= $0002;
  GFL_COLORS	= $0004;
  GFL_RGB	= $0010;
  GFL_RGBA	= $0020;
  GFL_BGR	= $0040;
  GFL_ABGR	= $0080;
  GFL_BGRA	= $0100;
  GFL_ARGB	= $0200;
  GFL_CMYK	= $0400;

  GFL_24BITS	= $1000;  // Only for gflBitmapTypeIsSupportedByIndex or gflBitmapTypeIsSupportedByName
  GFL_32BITS	= $2000;
  GFL_48BITS	= $4000;
  GFL_64BITS	= $8000;
type
  GFL_BITMAP_TYPE= GFL_UINT16;


// Color Palette
type
  PGFL_COLORMAP= ^GFL_COLORMAP;
  GFL_COLORMAP= record
    Red, Green, Blue,
    Alpha: array[0..255] of GFL_UINT8;
  end;

// Color
type
  PGFL_COLOR= ^GFL_COLOR;
  GFL_COLOR= record
    Red, Green, Blue,
    Alpha: GFL_UINT16;
  end;

// BITMAP
type
  PGFL_BITMAP= ^GFL_BITMAP;
  PPGFL_BITMAP= ^PGFL_BITMAP;
  GFL_BITMAP= record
    BType: GFL_BITMAP_TYPE;
    Origin: GFL_ORIGIN;
    Width, Height: GFL_INT32;
    BytesPerLine: GFL_UINT32;
    LinePadding: GFL_INT16;
    BitsPerComponent,   // 1, 8, 10, 12, 16
    ComponentsPerPixel, // 1, 3, 4
    BytesPerPixel,      // Only valid for 8 or more bits
    Xdpi, Ydpi: GFL_UINT16;
    TransparentIndex,    // -1 if not used
    Reserved: GFL_INT16;
    ColorUsed: GFL_INT32;
    ColorMap: PGFL_COLORMAP;
    Data: PGFL_UINT8;
    Comment: PChar;
    MetaData: Pointer;
    XOffset, YOffset: GFL_INT32;
    Name: PChar;
  end;

  // 2010-03-17 AmigoJack: do we need these Function (Macros) at all somewhere?
  {function GFL_IS24BITS( bitmap: PGFL_Bitmap ): Boolean;
  function GFL_IS32BITS( bitmap: PGFL_Bitmap ): Boolean;
  function GFL_ISTRUECOLORS( bitmap: PGFL_Bitmap ): Boolean;}
  function GFL_ISTRUECOLORS( bitmap: PGFL_Bitmap ): Boolean;


// Channels Order
const
  GFL_CORDER_INTERLEAVED= 0;
  GFL_CORDER_SEQUENTIAL	= 1;
  GFL_CORDER_SEPARATE	= 2;
type
  GFL_CORDER= GFL_UINT16;


// Channels Type
const
  GFL_CTYPE_GREYSCALE	= 0;
  GFL_CTYPE_RGB		= 1;
  GFL_CTYPE_BGR		= 2;
  GFL_CTYPE_RGBA	= 3;
  GFL_CTYPE_ABGR	= 4;
  GFL_CTYPE_CMY		= 5;
  GFL_CTYPE_CMYK	= 6;
type
  GFL_CTYPE= GFL_UINT16;


// Lut Type (For DPX/Cineon)
const
  GFL_LUT_TO8BITS	= 1;
  GFL_LUT_TO10BITS	= 2;
  GFL_LUT_TO12BITS	= 3;
  GFL_LUT_TO16BITS	= 4;
type
  GFL_LUT_TYPE= GFL_UINT16;


// Callbacks
type
  GFL_ALLOC_CALLBACK= function( size_t: GFL_UINT32; user_parms: Pointer ): Pointer; stdcall;
  GFL_REALLOC_CALLBACK= function( ptr: Pointer; new_size: GFL_UINT32; user_parms: Pointer ): Pointer; stdcall;
  GFL_FREE_CALLBACK= procedure( ptr, user_parms: Pointer ); stdcall;

  GFL_HANDLE= Pointer;

  GFL_READ_CALLBACK= function( handle: GFL_HANDLE; var buffer; size: GFL_UINT32 ): GFL_UINT32; stdcall;
  GFL_TELL_CALLBACK= function( handle: GFL_HANDLE ): GFL_UINT32; stdcall;
  GFL_SEEK_CALLBACK= function( handle: GFL_HANDLE; offset, origin: GFL_INT32 ): GFL_UINT32; stdcall;
  GFL_WRITE_CALLBACK= function( handle: GFL_HANDLE; buffer: Pointer; size: GFL_UINT32 ): GFL_UINT32; stdcall;

  GFL_ALLOCATEBITMAP_CALLBACK= function( width, height, number_component, bits_per_component,
    padding, bytes_per_line: GFL_INT32; user_params: Pointer ): Pointer; stdcall;
  GFL_PROGRESS_CALLBACK= procedure( percent: GFL_INT32; user_params: Pointer ); stdcall;
  GFL_WANTCANCEL_CALLBACK= function( user_params: Pointer ): GFL_BOOL; stdcall;

  GFL_VIRTUAL_SAVE_CALLBACK= function( var pBuffer: Pointer; line, nbline: GFL_INT32;
    user_params: Pointer ): GFL_ERROR; stdcall;

  GFL_LOAD_INFO_CB= record
    BType: GFL_BITMAP_TYPE;
    Origin: GFL_ORIGIN;
    Width, Height: GFL_INT32;
    BytesPerLine: GFL_UINT32;
    LinePadding: GFL_INT16;
    BitsPerComponent,
    ComponentsPerPixel,
    BytesPerPixel: GFL_UINT16;
  end;

  GFL_VIRTUAL_LOAD_CALLBACK= function( pBuffer: Pointer; line, nbline: GFL_INT32;
    var info: GFL_LOAD_INFO_CB; user_params: Pointer ): GFL_ERROR; stdcall;


// LOAD_PARAMS Flags
const
  GFL_LOAD_SKIP_ALPHA        		= $0001;
  GFL_LOAD_IGNORE_READ_ERROR 		= $0002;
  GFL_LOAD_BY_EXTENSION_ONLY 		= $0004;
  GFL_LOAD_READ_ALL_COMMENT  		= $0008;
  GFL_LOAD_FORCE_COLOR_MODEL 		= $0010;
  GFL_LOAD_PREVIEW_NO_CANVAS_RESIZE 	= $0020;
  GFL_LOAD_BINARY_AS_GREY      		= $0040;
  GFL_LOAD_ORIGINAL_COLORMODEL 		= $0080;
  GFL_LOAD_ONLY_FIRST_FRAME    		= $0100;
  GFL_LOAD_ORIGINAL_DEPTH      		= $0200;
  GFL_LOAD_METADATA            		= $0400;
  GFL_LOAD_COMMENT             		= $0800;
  GFL_LOAD_HIGH_QUALITY_THUMBNAIL 	= $1000;
  GFL_LOAD_EMBEDDED_THUMBNAIL     	= $2000;
  GFL_LOAD_ORIENTED_THUMBNAIL     	= $4000;
  GFL_LOAD_ORIGINAL_EMBEDDED_THUMBNAIL  = $8000;
  GFL_LOAD_ORIENTED                     = $8000;


// GFL_LOAD_CALLBACKS struct
type
  PGFL_LOAD_CALLBACKS= ^GFL_LOAD_CALLBACKS;
  GFL_LOAD_CALLBACKS= record
    Read: GFL_READ_CALLBACK;
    Tell: GFL_TELL_CALLBACK;
    Seek: GFL_SEEK_CALLBACK;

    AllocateBitmap: GFL_ALLOCATEBITMAP_CALLBACK;
    AllocateBitmapParams: Pointer;

    Progress: GFL_PROGRESS_CALLBACK;
    ProgressParams: Pointer;

    WantCancel: GFL_WANTCANCEL_CALLBACK;
    WantCancelParams: Pointer;

    SetLine: GFL_VIRTUAL_LOAD_CALLBACK;
    SetLineParams: Pointer;
  end;


// LOAD_PARAMS
type
  PGFL_LOAD_PARAMS= ^GFL_LOAD_PARAMS;
  GFL_LOAD_PARAMS= record
    Flags: GFL_UINT32;
    FormatIndex,                 // -1 for automatic recognition
    ImageWanted: GFL_INT32;      // for multi-page or animated file
    Origin: GFL_ORIGIN;          // default: GFL_TOP_LEFT
    ColorModel: GFL_BITMAP_TYPE; // Only for 24/32 bits picture, GFL_RGB/GFL_RGBA (default), GFL_BGR/GFL_ABGR, GFL_BGRA, GFL_ARGB
    LinePadding: GFL_UINT32;     // 1, 2, 4, ...
    DefaultAlpha: GFL_UINT8;     // Used if alpha doesn't exist in original file & ColorModel=RGBA/BGRA/ABGR/ARGB

    PsdNoAlphaForNonLayer,
    PngComposeWithAlpha,
    WMFHighResolution: GFL_UINT8;

    // RAW/YUV only
    Width, Height: GFL_INT32;
    Offset: GFL_UINT32;

    // RAW only
    ChannelOrder: GFL_CORDER;
    ChannelType: GFL_CTYPE;

    // PCD only
    PcdBase: GFL_UINT16; // PCD -> 2:768x576, 1:384x288, 0:192x144

    // EPS/PS/AI/PDF only
    EpsDpi: GFL_UINT16;
    EpsWidth, EpsHeight: GFL_INT32;

    // DPX/Cineon only
    LutType: GFL_LUT_TYPE; // GFL_LUT_TO8BITS, GFL_LUT_TO10BITS, GFL_LUT_TO12BITS, GFL_LUT_TO16BITS
    Reserved3: GFL_UINT16;
    LutData: PGFL_UINT16; // RRRR.../GGGG..../BBBB.....
    LutFilename: PChar;

    // Camera RAW only
    CameraRawUseAutomaticBalance,
    CameraRawUseCameraBalance,
    CameraRawHighlight,
    Reserved4: GFL_UINT8;
    CameraRawGamma,
    CameraRawBrightness,
    CameraRawRedScaling,
    CameraRawBlueScaling: Single;

    Callbacks: GFL_LOAD_CALLBACKS;

    UserParams: Pointer;
  end;


// SAVE_PARAMS
const
  GFL_SAVE_REPLACE_EXTENSION	= $0001;
  GFL_SAVE_WANT_FILENAME	= $0002;
  GFL_SAVE_ANYWAY		= $0004;
  GFL_SAVE_ICC_PROFILE		= $0008;  // Currently only available for jpeg

  GFL_BYTE_ORDER_DEFAULT	= 0;
  GFL_BYTE_ORDER_LSBF		= 1;
  GFL_BYTE_ORDER_MSBF		= 2;

type
  GFL_SaveCallbacks= record
    Write: GFL_WRITE_CALLBACK;
    Tell: GFL_TELL_CALLBACK;
    Seek: GFL_SEEK_CALLBACK;

    GetLine: GFL_VIRTUAL_SAVE_CALLBACK;
    GetLineParams: Pointer;
  end;

  PGFL_SAVE_PARAMS= ^GFL_SAVE_PARAMS;
  GFL_SAVE_PARAMS= record
    Flags: GFL_UINT32;
    FormatIndex: GFL_INT32;

    Compression: GFL_COMPRESSION;
    Quality,                        // Jpeg + Wic + Fpx
    CompressionLevel: GFL_INT16;    // Png
    Interlaced,                     // Gif
    Progressive,                    // Jpeg
    OptimizeHuffmanTable,           // Jpeg
    InAscii: GFL_BOOL;              // PPM

    // DPX/Cineon only
    LutType: GFL_LUT_TYPE;  // GFL_LUT_TO8BITS, GFL_LUT_TO10BITS, GFL_LUT_TO12BITS, GFL_LUT_TO16BITS

    DpxByteOrder,
    CompressRatio: GFL_UINT8;  // JPEG2000
    MaxFileSize: GFL_UINT32;  // JPEG2000

    LutData: PGFL_UINT16;  // RRRR.../GGGG..../BBBB.....
    LutFilename: PChar;

    // For RAW/YUV
    Offset: GFL_UINT32;
    ChannelOrder: GFL_CORDER;
    ChannelType: GFL_CTYPE;

    Callbacks: GFL_SaveCallbacks;

    UserParams: Pointer;
  end;


// Color model
const
  GFL_CM_RGB	= 0;
  GFL_CM_GREY	= 1;
  GFL_CM_CMY	= 2;
  GFL_CM_CMYK	= 3;
  GFL_CM_YCBCR	= 4;
  GFL_CM_YUV16	= 5;
  GFL_CM_LAB	= 6;
  GFL_CM_LOGLUV	= 7;
  GFL_CM_LOGL	= 8;
type
  GFL_COLORMODEL= GFL_UINT16;


// FILE_INFORMATION
type
  PGFL_FILE_INFORMATION= ^GFL_FILE_INFORMATION;
  GFL_FILE_INFORMATION= record
    BType: GFL_BITMAP_TYPE; // Not used
    Origin: GFL_ORIGIN;
    Width, Height,
    FormatIndex: GFL_INT32;
    FormatName: array[0..7] of Char;
    Description: array[0..63] of Char;
    Xdpi, Ydpi,
    BitsPerComponent,
    ComponentsPerPixel: GFL_UINT16;
    NumberOfImages: GFL_INT32;
    FileSize: GFL_UINT32;
    ColorModel: GFL_COLORMODEL;
    Compression: GFL_COMPRESSION;
    CompressionDescription: array[0..63] of Char;
    XOffset,
    YOffset: GFL_INT32;

    ExtraInfos: Pointer;
  end;

const
  GFL_READ	= $01;
  GFL_WRITE	= $02;


// FORMAT_INFORMATION
type
  PGFL_FORMAT_INFORMATION= ^GFL_FORMAT_INFORMATION;
  GFL_FORMAT_INFORMATION= record
    Index: GFL_INT32;
    Name: array[0..7] of Char;
    Description: array[0..63] of Char;
    Status,
    NumberOfExtension: GFL_UINT32;
    Extension: array[0..15, 0..7] of Char;
  end;


// IPTC
type
  PGFL_IPTC_ENTRY= ^GFL_IPTC_ENTRY;
  GFL_IPTC_ENTRY= packed record
    Id: GFL_UINT32;
    Name, Value: PChar;
  end;

  PGFL_IPTC_DATA= ^GFL_IPTC_DATA;
  GFL_IPTC_DATA= packed record
    NumberOfItems: GFL_UINT32;
    ItemsList: PGFL_IPTC_ENTRY;
  end;

const
  GFL_IPTC_BYLINE		= $50;
  GFL_IPTC_BYLINETITLE		= $55;
  GFL_IPTC_CREDITS		= $6e;
  GFL_IPTC_SOURCE		= $73;
  GFL_IPTC_CAPTIONWRITER	= $7a;
  GFL_IPTC_CAPTION		= $78;
  GFL_IPTC_HEADLINE	 	= $69;
  GFL_IPTC_SPECIALINSTRUCTIONS	= $28;
  GFL_IPTC_OBJECTNAME		= $05;
  GFL_IPTC_DATECREATED		= $37;
  GFL_IPTC_RELEASEDATE		= $1e;
  GFL_IPTC_TIMECREATED		= $3c;
  GFL_IPTC_RELEASETIME		= $23;
  GFL_IPTC_CITY			= $5a;
  GFL_IPTC_STATE		= $5f;
  GFL_IPTC_COUNTRY 		= $65;
  GFL_IPTC_COUNTRYCODE		= $64;
  GFL_IPTC_SUBLOCATION		= $5c;
  GFL_IPTC_ORIGINALTRREF	= $67;
  GFL_IPTC_CATEGORY		= $0f;
  GFL_IPTC_COPYRIGHT		= $74;
  GFL_IPTC_EDITSTATUS		= $07;
  GFL_IPTC_PRIORITY		= $0a;
  GFL_IPTC_OBJECTCYCLE 		= $4b;
  GFL_IPTC_JOBID 		= $16;
  GFL_IPTC_PROGRAM		= $41;
  GFL_IPTC_KEYWORDS		= $19;
  GFL_IPTC_SUPCATEGORIES	= $14;
  GFL_IPTC_CONTENT_LOCATION	= $1b;
  GFL_IPTC_PROGRAM_VERSION	= $46;
  GFL_IPTC_CONTACT		= $76;


// EXIF
const
  GFL_EXIF_IFD_0                = $0001;  // 2010-03-15 Miguel Angel
  GFL_EXIF_MAIN_IFD             = $0002;  // _IFD_0 and _MAIN_IFD were mixed up
  GFL_EXIF_INTEROPERABILITY_IFD = $0004;
  GFL_EXIF_IFD_THUMBNAIL        = $0008;
  GFL_EXIF_GPS_IFD              = $0010;
  GFL_EXIF_MAKERNOTE_IFD        = $0020;

  GFL_EXIF_MAKER		= $010F;
  GFL_EXIF_MODEL		= $0110;
  GFL_EXIF_ORIENTATION		= $0112;
  GFL_EXIF_EXPOSURETIME		= $829A;
  GFL_EXIF_FNUMBER		= $829D;
  GFL_EXIF_DATETIME_ORIGINAL 	= $9003;
  GFL_EXIF_SHUTTERSPEED		= $9201;
  GFL_EXIF_APERTURE		= $9202;
  GFL_EXIF_MAXAPERTURE		= $9205;
  GFL_EXIF_FOCALLENGTH		= $920A;

type
  PGFL_EXIF_ENTRY= ^GFL_EXIF_ENTRY;
  GFL_EXIF_ENTRY= packed record
    Flag,  // EXIF_...IFD
    Tag: GFL_UINT32;
    Name, Value: PChar;
  end;

  PGFL_EXIF_DATA= ^GFL_EXIF_DATA;
  GFL_EXIF_DATA= packed record
    NumberOfItems: GFL_UINT32;
    ItemsList: PGFL_EXIF_ENTRY;
  end;

// For Advanced developer only
const
  GFL_EXIF_BYTE      = 1;
  GFL_EXIF_STRING    = 2;
  GFL_EXIF_USHORT    = 3;
  GFL_EXIF_ULONG     = 4;
  GFL_EXIF_URATIONAL = 5;
  GFL_EXIF_SBYTE     = 6;
  GFL_EXIF_UNDEFINED = 7;
  GFL_EXIF_SSHORT    = 8;
  GFL_EXIF_SLONG     = 9;
  GFL_EXIF_SRATIONAL = 10;
  GFL_EXIF_SINGLEF   = 11;
  GFL_EXIF_DOUBLE    = 12;

type
   PGFL_EXIF_ENTRYEX= ^GFL_EXIF_ENTRYEX;
   GFL_EXIF_ENTRYEX= packed record
     Tag,
     Format: GFL_UINT16;
     Ifd,
     NumberOfComponents: GFL_INT32;
     Value: GFL_UINT32;
     DataLength: GFL_INT32;
     Data: Pointer;

     Next: PGFL_EXIF_ENTRYEX;
   end;

  PGFL_EXIF_DATAEX= ^GFL_EXIF_DATAEX;
  GFL_EXIF_DATAEX= packed record
    Root: PGFL_EXIF_ENTRYEX;
    UseMsbf: GFL_INT32;
  end;


{$IFDEF STATIC}  // 2010-03-09 AmigoJack
  // functions in LibGflxxx.dll

  // Common Memory Handling
  function gflMemoryAlloc( size: GFL_UINT32): Pointer; stdcall;
  function gflMemoryRealloc( ptr: Pointer; size: GFL_UINT32 ): Pointer; stdcall;
  procedure gflMemoryFree( ptr: Pointer ); stdcall;

  // Version Info
  function gflGetVersion: PChar; stdcall;
  function gflGetVersionOfLibformat: PChar; stdcall;

  // Initialization
  function gflLibraryInit: GFL_ERROR; stdcall;
  function gflLibraryInitEx( alloc_callback: GFL_ALLOC_CALLBACK;
    realloc_callback: GFL_REALLOC_CALLBACK; free_callback: GFL_FREE_CALLBACK;
    user_parms: Pointer ): GFL_ERROR; stdcall;
  procedure gflLibraryExit; stdcall;
  procedure gflEnableLZW( value: GFL_BOOL ); stdcall;
  procedure gflSetPluginsPathname( const pathname: PChar ); stdcall;

  // Infos of supported Formats
  function gflGetNumberOfFormat: GFL_INT32; stdcall;
  function gflGetFormatIndexByName( const name: PChar ): GFL_INT32; stdcall;
  function gflGetFormatNameByIndex( index: GFL_INT32 ): PChar; stdcall;
  function gflFormatIsSupported( const name: PChar ): GFL_BOOL; stdcall;
  function gflFormatIsWritableByIndex( index: GFL_INT32 ): GFL_BOOL; stdcall;
  function gflFormatIsWritableByName( const name: PChar ): GFL_BOOL; stdcall;
  function gflFormatIsReadableByIndex( index: GFL_INT32 ): GFL_BOOL; stdcall;
  function gflFormatIsReadableByName( const name: PChar ): GFL_BOOL; stdcall;
  function gflGetDefaultFormatSuffixByIndex( index: GFL_INT32 ): PChar; stdcall;
  function gflGetDefaultFormatSuffixByName( const name: PChar ): PChar; stdcall;
  function gflGetFormatDescriptionByIndex( index: GFL_INT32 ): PChar; stdcall;
  function gflGetFormatDescriptionByName( const name: PChar ): PChar; stdcall;
  function gflGetFormatInformationByIndex( index: GFL_INT32; var info: GFL_FORMAT_INFORMATION ): GFL_ERROR; stdcall;
  function gflGetFormatInformationByName( const name: PChar; var info: GFL_FORMAT_INFORMATION ): GFL_ERROR; stdcall;
{$ELSE}  // 2010-03-09 AmigoJack (until ENDIF)
var
  // Common Memory Handling
  gflMemoryAlloc: function( size: GFL_UINT32 ): Pointer; stdcall;
  gflMemoryRealloc: function( ptr: Pointer; size: GFL_UINT32 ): Pointer; stdcall;
  gflMemoryFree: procedure( ptr: Pointer ); stdcall;

  // Version Info
  gflGetVersion: function(): PChar; stdcall;
  gflGetVersionOfLibformat: function(): PChar; stdcall;

  // Initialization
  gflLibraryInit: function(): GFL_ERROR; stdcall;
  gflLibraryInitEx: function( alloc_callback: GFL_ALLOC_CALLBACK;
    realloc_callback: GFL_REALLOC_CALLBACK; free_callback: GFL_FREE_CALLBACK;
    user_parms: Pointer ): GFL_ERROR; stdcall;
  gflLibraryExit: procedure; stdcall;
  gflEnableLZW: procedure( value: GFL_BOOL ); stdcall;
  gflSetPluginsPathname: procedure( const pathname: PChar ); stdcall;

  // Infos of supported Formats
  gflGetNumberOfFormat: function(): GFL_INT32; stdcall;
  gflGetFormatIndexByName: function( const name: PChar ): GFL_INT32; stdcall;
  gflGetFormatNameByIndex: function( index: GFL_INT32 ): PChar; stdcall;
  gflFormatIsSupported: function( const name: PChar ): GFL_BOOL; stdcall;
  gflFormatIsWritableByIndex: function( index: GFL_INT32 ): GFL_BOOL; stdcall;
  gflFormatIsWritableByName: function( const name: PChar ): GFL_BOOL; stdcall;
  gflFormatIsReadableByIndex: function( index: GFL_INT32 ): GFL_BOOL; stdcall;
  gflFormatIsReadableByName: function( const name: PChar ): GFL_BOOL; stdcall;
  gflGetDefaultFormatSuffixByIndex: function( index: GFL_INT32 ): PChar; stdcall;
  gflGetDefaultFormatSuffixByName: function( const name: PChar ): PChar; stdcall;
  gflGetFormatDescriptionByIndex: function( index: GFL_INT32 ): PChar; stdcall;
  gflGetFormatDescriptionByName: function( const name: PChar ): PChar; stdcall;
  gflGetFormatInformationByIndex: function( index: GFL_INT32; var info: GFL_FORMAT_INFORMATION ): GFL_ERROR; stdcall;
  gflGetFormatInformationByName: function( const name: PChar; var info: GFL_FORMAT_INFORMATION ): GFL_ERROR; stdcall;
{$ENDIF}  // 2010-03-09 AmigoJack (from previous IFDEF STATIC ... ELSE)


// Info of supported save params
const
  GFL_SAVE_PARAMS_QUALITY		= 0;  // 0<= quality<= 100
  GFL_SAVE_PARAMS_COMPRESSION_LEVEL	= 1;  // 0<= level<= 9
  GFL_SAVE_PARAMS_INTERLACED		= 2;
  GFL_SAVE_PARAMS_PROGRESSIVE		= 3;
  GFL_SAVE_PARAMS_OPTIMIZE_HUFFMAN	= 4;
  GFL_SAVE_PARAMS_IN_ASCII		= 5;
  GFL_SAVE_PARAMS_LUT			= 6;
type
  GFL_SAVE_PARAMS_TYPE= GFL_UINT32;


{$IFDEF STATIC}  // 2010-03-09 AmigoJack
  function gflSaveParamsIsSupportedByIndex( index: GFL_INT32; params_type: GFL_SAVE_PARAMS_TYPE ): GFL_Bool; stdcall;
  function gflSaveParamsIsSupportedByName( const name: PChar; params_type: GFL_SAVE_PARAMS_TYPE ): GFL_Bool; stdcall;
  function gflCompressionIsSupportedByIndex( index: GFL_INT32; comp: GFL_COMPRESSION ): GFL_Bool; stdcall;
  function gflCompressionIsSupportedByName( const name: PChar; comp: GFL_COMPRESSION ): GFL_Bool; stdcall;
  function gflBitmapIsSupportedByIndex( index: GFL_INT32; const bitmap: PGFL_BITMAP ): GFL_Bool; stdcall;
  function gflBitmapIsSupportedByName( const name: PChar; const bitmap: PGFL_BITMAP ): GFL_Bool; stdcall;
  function gflBitmapTypeIsSupportedByIndex( index: GFL_INT32; btype: GFL_BITMAP_TYPE;
    bits_per_component: GFL_UINT16 ): GFL_Bool; stdcall;
  function gflBitmapTypeIsSupportedByName( const name: PChar; btype: GFL_BITMAP_TYPE;
    bits_per_component: GFL_UINT16 ): GFL_Bool; stdcall;

  function gflGetErrorString( error: GFL_ERROR ): PChar; stdcall;
  function gflGetLabelForColorModel( color_model: GFL_COLORMODEL ): PChar; stdcall;

  function gflGetFileInformation( const filename: PChar; index: GFL_INT32;
    var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  function gflGetFileInformationFromHandle( handle: GFL_HANDLE; index: GFL_INT32;
    const callbacks: PGFL_LOAD_CALLBACKS; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  function gflGetFileInformationFromMemory( const data: PGFL_UINT8; data_length: GFL_UINT32;
    index: GFL_INT32; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  procedure gflFreeFileInformation( var info: GFL_FILE_INFORMATION ); stdcall;

  // Loading and Saving
  procedure gflGetDefaultLoadParams( var params: GFL_LOAD_PARAMS ); stdcall;
  function gflLoadBitmap( const filename: PChar; bitmap: PPGFL_BITMAP;
    const params: PGFL_LOAD_PARAMS; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  function gflLoadBitmapFromHandle( handle: GFL_HANDLE; bitmap: PPGFL_BITMAP;
    const params: PGFL_LOAD_PARAMS; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  function gflLoadBitmapFromMemory( const data: PGFL_UINT8; data_length: GFL_UINT32;
    bitmap: PPGFL_BITMAP; const params: PGFL_LOAD_PARAMS; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;

  procedure gflGetDefaultThumbnailParams( var params: GFL_LOAD_PARAMS ); stdcall;
  function gflLoadThumbnail( const filename: PChar; width, height: GFL_INT32; bitmap: PPGFL_BITMAP;
    const params: PGFL_LOAD_PARAMS; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  function gflLoadThumbnailFromHandle( handle: GFL_HANDLE; width, height: GFL_INT32;
    bitmap: PPGFL_BITMAP; const params: PGFL_LOAD_PARAMS; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  function gflLoadThumbnailFromMemory( const data: PGFL_UINT8; data_length: GFL_UINT32;
    width, height: GFL_INT32; bitmap: PPGFL_BITMAP; const params: PGFL_LOAD_PARAMS;
    var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;

  procedure gflGetDefaultSaveParams( var params: GFL_SAVE_PARAMS ); stdcall;
  function gflSaveBitmap( filename: PChar; const bitmap: PGFL_BITMAP;
    const params: PGFL_SAVE_PARAMS ): GFL_ERROR; stdcall;
  function gflSaveBitmapIntoHandle( handle: GFL_HANDLE; const bitmap: PGFL_BITMAP;
    const params: PGFL_SAVE_PARAMS ): GFL_ERROR; stdcall;
  function gflSaveBitmapIntoMemory( var data: PGFL_UINT8; var data_length: GFL_UINT32;
    const bitmap: PGFL_BITMAP; const params: PGFL_SAVE_PARAMS ): GFL_ERROR; stdcall;


  {$IFDEF UNICODE_SUPPORT}
  procedure gflSetPluginsPathnameW( const S: PWideChar ); stdcall;
  function gflGetFileInformationW( const filename: PWideChar; index: GFL_INT32;
    var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  function gflLoadBitmapW( const filename: PWideChar; Bitmap: PPGFL_BITMAP;
    const params: PGFL_LOAD_PARAMS; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  function gflLoadThumbnailW( const filename: PWideChar; width, height: GFL_INT32;
    bitmap: PPGFL_BITMAP; const params: PGFL_LOAD_PARAMS; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  function gflSaveBitmapW( filename: PWideChar; const bitmap: PGFL_BITMAP;
    const params: PGFL_SAVE_PARAMS ): GFL_ERROR; stdcall;
  {$ENDIF}
{$ELSE}  // 2010-03-09 AmigoJack (until ENDIF)
var
  gflSaveParamsIsSupportedByIndex: function( index: GFL_INT32; params_type: GFL_SAVE_PARAMS_TYPE ): GFL_Bool; stdcall;
  gflSaveParamsIsSupportedByName: function( const name: PChar; params_type: GFL_SAVE_PARAMS_TYPE ): GFL_Bool; stdcall;
  gflCompressionIsSupportedByIndex: function( index: GFL_INT32; comp: GFL_COMPRESSION ): GFL_Bool; stdcall;
  gflCompressionIsSupportedByName: function( const name: PChar; comp: GFL_COMPRESSION ): GFL_Bool; stdcall;
  gflBitmapIsSupportedByIndex: function( index: GFL_INT32; const bitmap: PGFL_BITMAP ): GFL_Bool; stdcall;
  gflBitmapIsSupportedByName: function( const name: PChar; const bitmap: PGFL_BITMAP ): GFL_Bool; stdcall;
  gflBitmapTypeIsSupportedByIndex: function( index: GFL_INT32; btype: GFL_BITMAP_TYPE;
    bits_per_component: GFL_UINT16 ): GFL_Bool; stdcall;
  gflBitmapTypeIsSupportedByName: function( const name: PChar; btype: GFL_BITMAP_TYPE;
    bits_per_component: GFL_UINT16 ): GFL_Bool; stdcall;

  gflGetErrorString: function( error: GFL_ERROR ): PChar; stdcall;
  gflGetLabelForColorModel: function( color_model: GFL_COLORMODEL ): PChar; stdcall;

  gflGetFileInformation: function( const filename: PChar; index: GFL_INT32; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  gflGetFileInformationFromHandle: function( handle: GFL_HANDLE; index: GFL_INT32;
    const callbacks: PGFL_LOAD_CALLBACKS; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  gflGetFileInformationFromMemory: function( const data: PGFL_UINT8; data_length: GFL_UINT32;
    index: GFL_INT32; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  gflFreeFileInformation: procedure( var info: GFL_FILE_INFORMATION ); stdcall;

  // Loading and Saving
  gflGetDefaultLoadParams: procedure( var params: GFL_LOAD_PARAMS ); stdcall;
  gflLoadBitmap: function( const filename: PChar; bitmap: PPGFL_BITMAP;
    const params: PGFL_LOAD_PARAMS; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  gflLoadBitmapFromHandle: function( handle: GFL_HANDLE; bitmap: PPGFL_BITMAP;
    const params: PGFL_LOAD_PARAMS; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  gflLoadBitmapFromMemory: function( const data: PGFL_UINT8; data_length: GFL_UINT32;
    bitmap: PPGFL_BITMAP; const params: PGFL_LOAD_PARAMS; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;

  gflGetDefaultThumbnailParams: procedure( var params: GFL_LOAD_PARAMS ); stdcall;
  gflLoadThumbnail: function( const filename: PChar; width, height: GFL_INT32; bitmap: PPGFL_BITMAP;
    const params: PGFL_LOAD_PARAMS; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  gflLoadThumbnailFromHandle: function( handle: GFL_HANDLE; width, height: GFL_INT32;
    bitmap: PPGFL_BITMAP; const params: PGFL_LOAD_PARAMS; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  gflLoadThumbnailFromMemory: function( const data: PGFL_UINT8; data_length: GFL_UINT32;
    width, height: GFL_INT32; bitmap: PPGFL_BITMAP; const params: PGFL_LOAD_PARAMS;
    var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;

  gflGetDefaultSaveParams: procedure( var params: GFL_SAVE_PARAMS ); stdcall;
  gflSaveBitmap: function( filename: PChar; const bitmap: PGFL_BITMAP;
    const params: PGFL_SAVE_PARAMS ): GFL_ERROR; stdcall;
  gflSaveBitmapIntoHandle: function( handle: GFL_HANDLE; const bitmap: PGFL_BITMAP;
    const params: PGFL_SAVE_PARAMS ): GFL_ERROR; stdcall;
  gflSaveBitmapIntoMemory: function( var data: PGFL_UINT8; data_length: PGFL_UINT32;
    const bitmap: PGFL_BITMAP; const params: PGFL_SAVE_PARAMS ): GFL_ERROR; stdcall;


  {$IFDEF UNICODE_SUPPORT}
  gflSetPluginsPathnameW: procedure( const S: PWideChar ); stdcall;
  gflGetFileInformationW: function( const filename: PWideChar; index: GFL_INT32;
    var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  gflLoadBitmapW: function( const filename: PWideChar; Bitmap: PPGFL_BITMAP;
    const params: PGFL_LOAD_PARAMS; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  gflLoadThumbnailW: function( const filename: PWideChar; width, height: GFL_INT32;
    bitmap: PPGFL_BITMAP; const params: PGFL_LOAD_PARAMS; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  gflSaveBitmapW: function( filename: PWideChar; const bitmap: PGFL_BITMAP;
    const params: PGFL_SAVE_PARAMS ): GFL_ERROR; stdcall;
  {$ENDIF}
{$ENDIF}  // 2010-03-09 AmigoJack (from previous IFDEF STATIC ... ELSE)


type
  GFL_FILE_HANDLE= Pointer;

{$IFDEF STATIC}  // 2010-03-09 AmigoJack
  function gflFileCreate( var handle: GFL_FILE_HANDLE; const filename: PChar; image_count: GFL_UINT32;
    const params: PGFL_SAVE_PARAMS ): GFL_ERROR; stdcall;
  function gflFileAddPicture( handle: GFL_FILE_HANDLE; const bitmap: PGFL_BITMAP ): GFL_ERROR; stdcall;
  procedure gflFileClose( handle: GFL_FILE_HANDLE ); stdcall;

  {$IFDEF UNICODE_SUPPORT}
  function gflFileCreateW( var handle: GFL_FILE_HANDLE; const filename: PWideChar;
    image_count: GFL_UINT32; const params: PGFL_SAVE_PARAMS ): GFL_ERROR; stdcall;
  {$ENDIF}


  // Bitmap Memory Handling
  function gflAllockBitmap( BType: GFL_BITMAP_TYPE; width, height: GFL_INT32;
    line_padding: GFL_UINT32; const color: PGFL_COLOR ): PGFL_BITMAP; stdcall;
  function gflAllockBitmapEx( BType: GFL_BITMAP_TYPE; width, height: GFL_INT32;
    bits_per_component: GFL_UINT16; padding: GFL_UINT32; const color: PGFL_COLOR ): PGFL_BITMAP; stdcall;

  procedure gflFreeBitmap( var bitmap: GFL_BITMAP ); stdcall;
  procedure gflFreeBitmapData( var bitmap: GFL_BITMAP ); stdcall;
  function gflCloneBitmap( const bitmap: PGFL_BITMAP ): PGFL_BITMAP; stdcall;
  procedure gflBitmapSetName( var bitmap: PGFL_BITMAP; const name: PChar ); stdcall;

  // 2010-03-11 AmigoJack
  function gflGetExtraInfosCount( const info: PGFL_FILE_INFORMATION ): GFL_INT32; stdcall;
  procedure gflGetExtraInfos( const info: PGFL_FILE_INFORMATION; idx: GFL_INT32;
    var name, value: PChar ); stdcall;

  function gflBitmapHasEXIF( const bitmap: PGFL_BITMAP ): GFL_BOOL; stdcall;
  function gflBitmapHasIPTC( const bitmap: PGFL_BITMAP ): GFL_BOOL; stdcall;
  function gflBitmapHasICCProfile( const bitmap: PGFL_BITMAP ): GFL_BOOL; stdcall;

  procedure gflBitmapRemoveEXIFThumbnail( var bitmap: GFL_BITMAP ); stdcall;
  procedure gflBitmapRemoveICCProfile( var bitmap: GFL_BITMAP ); stdcall;
  procedure gflBitmapGetICCProfile( const bitmap: PGFL_BITMAP; var pData: PGFL_UINT8;
    var pLength: GFL_UINT32 ); stdcall;  // pData must be freed by gflFreeMemory
  procedure gflBitmapCopyICCProfile( const bitmap_src: PGFL_BITMAP; var bitmap_dst: GFL_BITMAP ); stdcall;
  procedure gflBitmapRemoveMetaData( var bitmap: GFL_BITMAP ); stdcall;

  function gflBitmapGetXMP( const bitmap: PGFL_BITMAP; var pData: PGFL_UINT8;
    var pLength: GFL_UINT32 ): GFL_BOOL; stdcall;

  function gflLoadEXIF( const filename: PChar; flags: GFL_UINT32 ): PGFL_EXIF_DATA; stdcall;

  function gflBitmapGetIPTC( var bitmap: GFL_BITMAP ): PGFL_IPTC_DATA; stdcall;
  function gflBitmapGetIPTCValue( const bitmap: PGFL_BITMAP; id: GFL_UINT32; value: PChar;
    value_length: GFL_INT32 ): GFL_ERROR; stdcall;
  function gflNewIPTC(): PGFL_IPTC_DATA; stdcall;
  procedure gflFreeIPTC( var iptc_data: GFL_IPTC_DATA ); stdcall;
  function gflSetIPTCValue( var iptc_data: GFL_IPTC_DATA; id: GFL_UINT32; value: PChar ): GFL_ERROR; stdcall;
  function gflRemoveIPTCValue( var iptc_data: GFL_IPTC_DATA; id: GFL_UINT32 ): GFL_ERROR; stdcall;

  function gflLoadIPTC( const filename: PChar ): PGFL_IPTC_DATA; stdcall;
  function gflSaveIPTC( const filename: PChar; const iptc_data: PGFL_IPTC_DATA ): GFL_ERROR; stdcall;
  function gflBitmapSetIPTC( var bitmap: GFL_BITMAP; const iptc_data: PGFL_IPTC_DATA ): GFL_ERROR; stdcall;

  {$IFDEF UNICODE_SUPPORT}
  function gflLoadEXIFW( const filename: PWideChar; flags: GFL_UINT32 ): PGFL_EXIF_DATA; stdcall;
  function gflLoadIPTCW( const filename: PWideChar ): PGFL_IPTC_DATA; stdcall;
  function gflSaveIPTCW( const filename: PWideChar; const iptc_data: PGFL_IPTC_DATA ): GFL_ERROR; stdcall;
  {$ENDIF}
{$ELSE}  // 2010-03-09 AmigoJack (until ENDIF)
var
  gflFileCreate: function( var handle: GFL_FILE_HANDLE; const filename: PChar;
    image_count: GFL_UINT32; const params: PGFL_SAVE_PARAMS ): GFL_ERROR; stdcall;
  gflFileAddPicture: function( handle: GFL_FILE_HANDLE; const bitmap: PGFL_BITMAP ): GFL_ERROR; stdcall;
  gflFileClose: procedure( handle: GFL_FILE_HANDLE ); stdcall;

  {$IFDEF UNICODE_SUPPORT}
  gflFileCreateW: function( var handle: GFL_FILE_HANDLE; const filename: PWideChar;
    image_count: GFL_UINT32; const params: PGFL_SAVE_PARAMS ): GFL_ERROR; stdcall;
  {$ENDIF}


  // Bitmap Memory Handling
  gflAllockBitmap: function( BType: GFL_BITMAP_TYPE; width, height: GFL_INT32;
    line_padding: GFL_UINT32; const color: PGFL_COLOR ): PGFL_BITMAP; stdcall;
  gflAllockBitmapEx: function( BType: GFL_BITMAP_TYPE; width, height: GFL_INT32;
    bits_per_component: GFL_UINT16; padding: GFL_UINT32; const color: PGFL_COLOR ): PGFL_BITMAP; stdcall;

  gflFreeBitmap: procedure( var bitmap: GFL_BITMAP ); stdcall;
  gflFreeBitmapData: procedure( var bitmap: GFL_BITMAP ); stdcall;
  gflCloneBitmap: function( const bitmap: PGFL_BITMAP ): PGFL_BITMAP; stdcall;
  gflBitmapSetName: procedure( var bitmap: PGFL_BITMAP; const name: PChar ); stdcall;

  // 2010-03-11 AmigoJack
  gflGetExtraInfosCount: function( const info: PGFL_FILE_INFORMATION ): GFL_INT32; stdcall;
  gflGetExtraInfos: procedure( const info: PGFL_FILE_INFORMATION; idx: GFL_INT32;
    var name, value: PChar ); stdcall;

  gflBitmapHasEXIF: function( const bitmap: PGFL_BITMAP ): GFL_BOOL; stdcall;
  gflBitmapHasIPTC: function( const bitmap: PGFL_BITMAP ): GFL_BOOL; stdcall;
  gflBitmapHasICCProfile: function( const bitmap: PGFL_BITMAP ): GFL_BOOL; stdcall;

  gflBitmapRemoveEXIFThumbnail: procedure( var bitmap: PGFL_BITMAP ); stdcall;
  gflBitmapRemoveICCProfile: procedure( var bitmap: PGFL_BITMAP ); stdcall;
  gflBitmapGetICCProfile: procedure( const bitmap: PGFL_BITMAP; var pData: PGFL_UINT8;
    var pLength: GFL_UINT32); stdcall;  // pData must be freed by gflFreeMemory
  gflBitmapCopyICCProfile: procedure( const bitmap_src: PGFL_BITMAP; var bitmap_dst: GFL_BITMAP ); stdcall;
  gflBitmapRemoveMetaData: procedure( var bitmap: GFL_BITMAP ); stdcall;

  gflBitmapGetXMP: function( const bitmap: PGFL_BITMAP; var pData: PGFL_UINT8;
    var pLength: GFL_UINT32 ): GFL_BOOL; stdcall;

  gflLoadEXIF: function( const filename: PChar; flags: GFL_UINT32 ): PGFL_EXIF_DATA; stdcall;

  gflBitmapGetIPTC: function( var bitmap: GFL_BITMAP ): PGFL_IPTC_DATA; stdcall;
  gflBitmapGetIPTCValue: function( const bitmap: PGFL_BITMAP; id: GFL_UINT32; value: PChar;
    value_length: GFL_INT32 ): GFL_ERROR; stdcall;
  gflNewIPTC: function(): PGFL_IPTC_DATA; stdcall;
  gflFreeIPTC: procedure( var iptc_data: GFL_IPTC_DATA ); stdcall;
  gflSetIPTCValue: function( var iptc_data: GFL_IPTC_DATA; id: GFL_UINT32; value: PChar ): GFL_ERROR; stdcall;
  gflRemoveIPTCValue: function( var iptc_data: GFL_IPTC_DATA; id: GFL_UINT32 ): GFL_ERROR; stdcall;

  gflLoadIPTC: function( const filename: PChar ): PGFL_IPTC_DATA; stdcall;
  gflSaveIPTC: function( const filename: PChar; const iptc_data: PGFL_IPTC_DATA ): GFL_ERROR; stdcall;
  gflBitmapSetIPTC: function( var bitmap: GFL_BITMAP; const iptc_data: PGFL_IPTC_DATA ): GFL_ERROR; stdcall;

  {$IFDEF UNICODE_SUPPORT}
  gflLoadEXIFW: function( const filename: PWideChar; flags: GFL_UINT32 ): PGFL_EXIF_DATA; stdcall;
  gflLoadIPTCW: function( const filename: PWideChar ): PGFL_IPTC_DATA; stdcall;
  gflSaveIPTCW: function( const filename: PWideChar; const iptc_data: PGFL_IPTC_DATA ): GFL_ERROR; stdcall;
  {$ENDIF}
{$ENDIF}  // 2010-03-09 AmigoJack (from previous IFDEF STATIC ... ELSE)


const
  GFL_EXIF_WANT_MAKERNOTES= $0001;  // 2010-03-16 AmigoJack

{$IFDEF STATIC}  // 2010-03-09 AmigoJack
  function gflBitmapGetEXIF( var bitmap: GFL_BITMAP; flags: GFL_UINT32 ): PGFL_EXIF_DATA; stdcall;
  function gflBitmapGetEXIFValue( const bitmap: PGFL_BITMAP; tag: GFL_UINT32; value: PChar;
    value_length: GFL_INT32 ): GFL_ERROR; stdcall;
  procedure gflFreeEXIF( var exif_data: GFL_EXIF_DATA ); stdcall;
  procedure gflBitmapSetComment( var bitmap: GFL_BITMAP; const comment: PChar ); stdcall;

  function gflBitmapGetEXIF2( const bitmap: PGFL_BITMAP ): PGFL_EXIF_DATAEX; stdcall;
  procedure gflFreeEXIF2( var exif_data: GFL_EXIF_DATAEX ); stdcall;
  procedure gflBitmapSetEXIF2( var bitmap: GFL_BITMAP; const exif: PGFL_EXIF_DATAEX ); stdcall;
  procedure gflBitmapSetEXIFValueString2( var exif: GFL_EXIF_DATAEX; ifd, tag: GFL_UINT16;
    const value: PChar ); stdcall;
  procedure gflBitmapSetEXIFValueInt2( var exif: GFL_EXIF_DATAEX; ifd, tag: GFL_UINT16;
    format, value: GFL_UINT32 ); stdcall;
  procedure gflBitmapSetEXIFValueRational2( var exif: GFL_EXIF_DATAEX; ifd, tag: GFL_UINT16;
    p, q: GFL_UINT32 ); stdcall;
  procedure gflBitmapSetEXIFValueRationalArray2( var exif: GFL_EXIF_DATAEX; ifd, tag: GFL_UINT16;
    const pq: PGFL_UINT32; count: GFL_INT32 ); stdcall;  // 2010-03-15 Miguel Angel

  function gflJPEGGetComment( const filename: PChar; comment: PChar; max_size: Integer ): GFL_ERROR; stdcall;
  function gflJPEGSetComment( const filename, comment: PChar ): GFL_ERROR; stdcall;
  function gflPNGGetComment( const filename: PChar; comment: PChar; max_size: Integer ): GFL_ERROR; stdcall;
  function gflPNGSetComment( const filename, comment: PChar ): GFL_ERROR; stdcall;

  {$IFDEF UNICODE_SUPPORT}
  function gflJPEGGetCommentW( const filename: PWideChar; comment: PChar; max_size: Integer ): GFL_ERROR; stdcall;
  function gflJPEGSetCommentW( const filename: PWideChar; const comment: PChar ): GFL_ERROR; stdcall;
  function gflPNGGetCommentW( const filename: PWideChar; comment: PChar; max_size: Integer ): GFL_ERROR; stdcall;
  function gflPNGSetCommentW( const filename: PWideChar; const comment: PChar ): GFL_ERROR; stdcall;
  {$ENDIF}

  // DPX LUT
  function gflIsLutFile( const filename: PChar ): GFL_BOOL; stdcall;
  function gflIsCompatibleLutFile( const filename: PChar; const components_per_pixel,
    bits_per_component: GFL_INT32; var lut_type: GFL_LUT_TYPE ): GFL_BOOL; stdcall;
  function gflApplyLutFile( var bitmap_src: GFL_BITMAP; bitmap_dst: PPGFL_BITMAP;
    const filename: PChar; lut_type: GFL_LUT_TYPE ): GFL_ERROR; stdcall;
{$ELSE}  // 2010-03-09 AmigoJack (until ENDIF)
var
  gflBitmapGetEXIF: function( var bitmap: GFL_BITMAP; flags: GFL_UINT32 ): PGFL_EXIF_DATA; stdcall;
  gflBitmapGetEXIFValue: function( const bitmap: PGFL_BITMAP; tag: GFL_UINT32; value: PChar;
    value_length: GFL_INT32 ): GFL_ERROR; stdcall;
  gflFreeEXIF: procedure( var exif_data: GFL_EXIF_DATA ); stdcall;
  gflBitmapSetComment: procedure( var bitmap: GFL_BITMAP; const comment: PChar ); stdcall;

  gflBitmapGetEXIF2: function( const bitmap: PGFL_BITMAP ): PGFL_EXIF_DATAEX; stdcall;
  gflFreeEXIF2: procedure( var exif_data: GFL_EXIF_DATAEX ); stdcall;
  gflBitmapSetEXIF2: procedure( var bitmap: GFL_BITMAP; const exif: PGFL_EXIF_DATAEX ); stdcall;
  gflBitmapSetEXIFValueString2: procedure( var exif: GFL_EXIF_DATAEX; ifd, tag: GFL_UINT16;
    const value: PChar ); stdcall;
  gflBitmapSetEXIFValueInt2: procedure( var exif: GFL_EXIF_DATAEX; ifd, tag: GFL_UINT16;
    format, value: GFL_UINT32 ); stdcall;
  gflBitmapSetEXIFValueRational2: procedure( var exif: GFL_EXIF_DATAEX; ifd, tag: GFL_UINT16;
    p, q: GFL_UINT32 ); stdcall;
  gflBitmapSetEXIFValueRationalArray2: procedure( var exif: GFL_EXIF_DATAEX; ifd, tag: GFL_UINT16;
    const pq: PGFL_UINT32; count: GFL_INT32 ); stdcall;  // 2010-03-15 Miguel Angel

  gflJPEGGetComment: function( const filename: PChar; comment: PChar; max_size: Integer ): GFL_ERROR; stdcall;
  gflJPEGSetComment: function( const filename, comment: PChar ): GFL_ERROR; stdcall;
  gflPNGGetComment: function( const filename: PChar; comment: PChar; max_size: Integer ): GFL_ERROR; stdcall;
  gflPNGSetComment: function( const filename, PChar ): GFL_ERROR; stdcall;

  {$IFDEF UNICODE_SUPPORT}
  gflJPEGGetCommentW: function( const filename: PWideChar; comment: PChar; max_size: Integer ): GFL_ERROR; stdcall;
  gflJPEGSetCommentW: function( const filename: PWideChar; comment: PChar ): GFL_ERROR; stdcall;
  gflPNGGetCommentW: function( const filename: PWideChar; comment: PChar; max_size: Integer ): GFL_ERROR; stdcall;
  gflPNGSetCommentW: function( const filename: PWideChar; comment: PChar ): GFL_ERROR; stdcall;
  {$ENDIF}

  // DPX LUT
  gflIsLutFile: function( const filename: PChar ): GFL_BOOL; stdcall;
  gflIsCompatibleLutFile: function( const filename: PChar; const components_per_pixel,
    bits_per_component: GFL_INT32; var lut_type: GFL_LUT_TYPE ): GFL_BOOL; stdcall;
  gflApplyLutFile: function( var bitmap_src: GFL_BITMAP; bitmap_dst: PPGFL_BITMAP;
    const filename: PChar; lut_type: GFL_LUT_TYPE ): GFL_ERROR; stdcall;
{$ENDIF}  // 2010-03-09 AmigoJack (from previous IFDEF STATIC ... ELSE)


// Bitmap resize
const
  GFL_RESIZE_QUICK	= 0;
  GFL_RESIZE_BILINEAR	= 1;
  GFL_RESIZE_HERMITE	= 2;
  GFL_RESIZE_GAUSSIAN	= 3;
  GFL_RESIZE_BELL	= 4;
  GFL_RESIZE_BSPLINE	= 5;
  GFL_RESIZE_MITSHELL	= 6;
  GFL_RESIZE_LANCZOS	= 7;

{$IFDEF STATIC}  // 2010-03-09 AmigoJack
  function gflResize( var src: GFL_BITMAP; dst: PPGFL_BITMAP; width, height: GFL_INT32;
    method, flags: GFL_UINT32 ): GFL_ERROR; stdcall;
{$ELSE}  // 2010-03-09 AmigoJack (until ENDIF)
var
  gflResize: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; width, height: GFL_INT32;
    method, flags: GFL_UINT32 ): GFL_ERROR; stdcall;
{$ENDIF}  // 2010-03-09 AmigoJack (from previous IFDEF STATIC ... ELSE)


// doesn't work yet for more than 8bit per component
const
  GFL_MODE_TO_BINARY    = 1;
  GFL_MODE_TO_4GREY     = 2;
  GFL_MODE_TO_8GREY     = 3;
  GFL_MODE_TO_16GREY    = 4;
  GFL_MODE_TO_32GREY    = 5;
  GFL_MODE_TO_64GREY    = 6;
  GFL_MODE_TO_128GREY   = 7;
  GFL_MODE_TO_216GREY   = 8;
  GFL_MODE_TO_256GREY   = 9;
  GFL_MODE_TO_8COLORS   = 12;
  GFL_MODE_TO_16COLORS  = 13;
  GFL_MODE_TO_32COLORS  = 14;
  GFL_MODE_TO_64COLORS  = 15;
  GFL_MODE_TO_128COLORS = 16;
  GFL_MODE_TO_216COLORS = 17;
  GFL_MODE_TO_256COLORS = 18;
  GFL_MODE_TO_RGB       = 19;
  GFL_MODE_TO_RGBA      = 20;
  GFL_MODE_TO_BGR       = 21;
  GFL_MODE_TO_ABGR      = 22;
  GFL_MODE_TO_BGRA      = 23;
  GFL_MODE_TO_ARGB      = 24;
  GFL_MODE_TO_TRUECOLORS= GFL_MODE_TO_RGB;
type
  GFL_MODE= GFL_UINT16;


const
  GFL_MODE_NO_DITHER		= 0;
  GFL_MODE_PATTERN_DITHER	= 1;
  GFL_MODE_HALTONE45_DITHER	= 2; // Only with GFL_MODE_TO_BINARY
  GFL_MODE_HALTONE90_DITHER	= 3; // Only with GFL_MODE_TO_BINARY
  GFL_MODE_ADAPTIVE		= 4;
  GFL_MODE_FLOYD_STEINBERG	= 5; // Pierre says: Only with GFL_MODE_TO_BINARY
                                     // I tested with 8 and 4 Bit and test is OK ??
type
  GFL_MODE_PARAMS= GFL_UINT16;


  // 2010-03-17 AmigoJack: fixed translation
  function gflGetBitmapPtr( const _bitmap: GFL_Bitmap; const _y: Integer ): Pointer;

{$IFDEF STATIC}  // 2010-03-09 AmigoJack
  function gflChangeColorDepth( var src: GFL_BITMAP; dst: PPGFL_BITMAP; mode: GFL_MODE;
    params: GFL_MODE_PARAMS ): GFL_ERROR; stdcall;

  function gflGetColorAt( const src: PGFL_BITMAP; x, y: GFL_INT32; var color: GFL_COLOR ): GFL_ERROR; stdcall;
  function gflSetColorAt( var src: GFL_BITMAP; x, y: GFL_INT32; const color: PGFL_COLOR ): GFL_ERROR; stdcall;
  function gflGetColorAtEx( const src: PGFL_BITMAP; ptr: PGFL_UINT8; var color: GFL_COLOR ): GFL_ERROR; stdcall;
  function gflSetColorAtEx( var src: GFL_BITMAP; ptr: PGFL_UINT8; const color: PGFL_COLOR ): GFL_ERROR; stdcall;
{$ELSE}  // 2010-03-09 AmigoJack (until ENDIF)
var
  gflChangeColorDepth: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; mode: GFL_MODE;
    params: GFL_MODE_PARAMS ): GFL_ERROR; stdcall;

  gflGetColorAt: function( const src: PGFL_BITMAP; x, y: GFL_INT32; var color: GFL_COLOR ): GFL_ERROR; stdcall;
  gflSetColorAt: function( var src: GFL_BITMAP; x, y: GFL_INT32; const color: PGFL_COLOR ): GFL_ERROR; stdcall;
  gflGetColorAtEx: function( const src: PGFL_BITMAP; ptr: PGFL_UINT8; var color: GFL_COLOR ): GFL_ERROR; stdcall;
  gflSetColorAtEx: function( var src: GFL_BITMAP; ptr: PGFL_UINT8; const color: PGFL_COLOR ): GFL_ERROR; stdcall;
{$ENDIF}  // 2010-03-09 AmigoJack (from previous IFDEF STATIC ... ELSE)


type
  PGFL_RECT= ^GFL_RECT;
  GFL_RECT= record
    x, y, w, h: GFL_INT32;
  end;

  PGFL_Point= ^GFL_POINT;
  GFL_POINT= record
    x, y: GFL_INT32;
  end;


{$IFDEF STATIC}  // 2010-03-09 AmigoJack
  function gflFlipVertical( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflFlipHorizontal( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflCrop( var src: GFL_BITMAP; dst: PPGFL_BITMAP; const rect: PGFL_RECT ): GFL_ERROR; stdcall;
  function gflAutoCrop( var src: GFL_BITMAP; dst: PPGFL_BITMAP; const color: PGFL_COLOR;
    tolerance: GFL_INT32 ): GFL_ERROR; stdcall;
  function gflAutoCrop2( var src: GFL_BITMAP; dst: PPGFL_BITMAP; const color: PGFL_COLOR;
    tolerance: GFL_INT32 ): GFL_ERROR; stdcall;
{$ELSE}  // 2010-03-09 AmigoJack (until ENDIF)
var
  gflFlipVertical: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflFlipHorizontal: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflCrop: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; const rect: PGFL_RECT ): GFL_ERROR; stdcall;
  gflAutoCrop: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; const color: PGFL_COLOR;
    tolerance: GFL_INT32 ): GFL_ERROR; stdcall;
  gflAutoCrop2: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; const color: PGFL_COLOR;
    tolerance: GFL_INT32 ): GFL_ERROR; stdcall;
{$ENDIF}  // 2010-03-09 AmigoJack (from previous IFDEF STATIC ... ELSE)


const
  GFL_CANVASRESIZE_TOPLEFT	= 0;
  GFL_CANVASRESIZE_TOP		= 1;
  GFL_CANVASRESIZE_TOPRIGHT	= 2;
  GFL_CANVASRESIZE_LEFT		= 3;
  GFL_CANVASRESIZE_CENTER	= 4;
  GFL_CANVASRESIZE_RIGHT	= 5;
  GFL_CANVASRESIZE_BOTTOMLEFT	= 6;
  GFL_CANVASRESIZE_BOTTOM	= 7;
  GFL_CANVASRESIZE_BOTTOMRIGHT	= 8;
type
  GFL_CANVASRESIZE= GFL_UINT32;

{$IFDEF STATIC}  // 2010-03-09 AmigoJack
  function gflResizeCanvas( var src: GFL_BITMAP; dst: PPGFL_BITMAP; width, height: GFL_INT32;
    mode: GFL_CANVASRESIZE; const color: PGFL_COLOR ): GFL_ERROR; stdcall;
  function gflScaleToGrey( var src: GFL_BITMAP; dst: PPGFL_BITMAP;
    width, height: GFL_INT32 ): GFL_ERROR; stdcall;

  function gflRotate( var src: GFL_BITMAP; dst: PPGFL_BITMAP; angle: GFL_INT32;
    const background_color: PGFL_COLOR ): GFL_ERROR; stdcall;
  function gflRotateFine( var src: GFL_BITMAP; dst: PPGFL_BITMAP; angle: double;
    const background_color: PGFL_COLOR ): GFL_ERROR; stdcall;
  function gflReplaceColor( var src: PGFL_BITMAP; dst: PPGFL_BITMAP;
    const color, new_color: PGFL_COLOR; tolerance: GFL_INT32 ): GFL_ERROR; stdcall;

  function gflBitblt( const src: PGFL_BITMAP; const rect: PGFL_RECT;
    var dst: GFL_BITMAP; x_dest, y_dest: GFL_INT32 ): GFL_ERROR; stdcall;
  function gflBitbltEx( const src: PGFL_BITMAP; const rect: PGFL_RECT;
    var dst: GFL_BITMAP; x_dest, y_dest: GFL_INT32 ): GFL_ERROR; stdcall;
  function gflMerge( const src: array of PGFL_BITMAP; const origin: array of GFL_POINT;
    const opacity: array of GFL_UINT32; num_bitmap: GFL_INT32; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflCombineAlpha( var src: GFL_BITMAP; dst: PPGFL_BITMAP; const color: PGFL_COLOR ): GFL_ERROR; stdcall;
  function gflSetTransparentColor( var src: GFL_BITMAP; dst: PPGFL_BITMAP;
    const mask_color, back_color: PGFL_COLOR ): GFL_ERROR; stdcall;
{$ELSE}  // 2010-03-09 AmigoJack (until ENDIF)
var
  gflResizeCanvas: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; width, height: GFL_INT32;
    mode: GFL_CANVASRESIZE; const color: PGFL_COLOR ): GFL_ERROR; stdcall;
  gflScaleToGrey: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP;
    width, height: GFL_INT32 ): GFL_ERROR; stdcall;

  gflRotate: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; angle: GFL_INT32;
    const background_color: PGFL_COLOR ): GFL_ERROR; stdcall;
  gflRotateFine: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; angle: double;
    const background_color: PGFL_COLOR ): GFL_ERROR; stdcall;
  gflReplaceColor: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP;
    const color, new_color: PGFL_COLOR; tolerance: GFL_INT32 ): GFL_ERROR; stdcall;

  gflBitblt: function( const src: PGFL_BITMAP; const rect: PGFL_RECT;
    var dst: GFL_BITMAP; x_dest, y_dest: GFL_INT32 ): GFL_ERROR; stdcall;
  gflBitbltEx: function( const src: PGFL_BITMAP; const rect: PGFL_RECT;
    var dst: GFL_BITMAP; x_dest, y_dest: GFL_INT32 ): GFL_ERROR; stdcall;
  gflMerge: function( const src: array of PGFL_BITMAP; const origin: array of GFL_POINT;
    const opacity: array of GFL_UINT32; num_bitmap: GFL_INT32; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflCombineAlpha: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; const color: PGFL_COLOR ): GFL_ERROR; stdcall;
  gflSetTransparentColor: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP;
    const mask_color, back_color: PGFL_COLOR ): GFL_ERROR; stdcall;
{$ENDIF}  // 2010-03-09 AmigoJack (from previous IFDEF STATIC ... ELSE)


// functions in LibGflExxx.dll

{$IFDEF STATIC}  // 2010-03-09 AmigoJack
  function gflGetNumberOfColorsUsed( var src: GFL_BITMAP ): GFL_UINT32; stdcall;

  function gflNegative( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflBrightness( var src: GFL_BITMAP; dst: PPGFL_BITMAP; brightness: GFL_INT32 ): GFL_ERROR; stdcall;
  function gflContrast( var src: GFL_BITMAP; dst: PPGFL_BITMAP; contrast: GFL_INT32 ): GFL_ERROR; stdcall;
  function gflGamma( var src: GFL_BITMAP; dst: PPGFL_BITMAP; gamma: double ): GFL_ERROR; stdcall;
  function gflLogCorrection( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflNormalize( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflEqualize( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflEqualizeOnLuminance( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflBalance( var src: GFL_BITMAP; dst: PPGFL_BITMAP; const color: PGFL_COLOR ): GFL_ERROR; stdcall;
  function gflAdjust( var src: GFL_BITMAP; dst: PPGFL_BITMAP; brightness, contrast: GFL_INT32;
    gamma: double ): GFL_ERROR; stdcall;
  function gflAdjustHLS( var src: GFL_BITMAP; bdst: PPGFL_BITMAP;
    hue, lightness, saturation: GFL_INT32 ): GFL_ERROR; stdcall;
  function gflAutomaticContrast( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflAutomaticLevels( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;

  function gflAverage( var src: GFL_BITMAP; dst: PPGFL_BITMAP; filter_size: GFL_INT32 ): GFL_ERROR; stdcall;
  function gflSoften( var src: GFL_BITMAP; dst: PPGFL_BITMAP; percentage: GFL_INT32 ): GFL_ERROR; stdcall;
  function gflBlur( var src: GFL_BITMAP; dst: PPGFL_BITMAP; percentage: GFL_INT32 ): GFL_ERROR; stdcall;
  function gflGaussianBlur( var src: GFL_BITMAP; dst: PPGFL_BITMAP; filter_size: GFL_INT32 ): GFL_ERROR; stdcall;
  function gflMaximum( var src: GFL_BITMAP; dst: PPGFL_BITMAP; filter_size: GFL_INT32 ): GFL_ERROR; stdcall;
  function gflMinimum( var src: GFL_BITMAP; dst: PPGFL_BITMAP; filter_size: GFL_INT32 ): GFL_ERROR; stdcall;
  function gflMedianBox( var src: GFL_BITMAP; dst: PPGFL_BITMAP; filter_size: GFL_INT32 ): GFL_ERROR; stdcall;
  function gflMedianCross( var src: GFL_BITMAP; dst: PPGFL_BITMAP; filter_size: GFL_INT32 ): GFL_ERROR; stdcall;
  function gflSharpen( var src: GFL_BITMAP; dst: PPGFL_BITMAP; percentage: GFL_INT32 ): GFL_ERROR; stdcall;

  function gflEnhanceDetail( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflEnhanceFocus( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflFocusRestoration( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflEdgeDetectLight( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflEdgeDetectMedium( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflEdgeDetectHeavy( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflEmboss( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflEmbossMore( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;

  function gflSepia( var src: GFL_BITMAP; bdst: PPGFL_BITMAP; percent: GFL_INT32 ): GFL_ERROR; stdcall;
  function gflSepiaEx( var src: GFL_BITMAP; bdst: PPGFL_BITMAP; percent: GFL_INT32;
    const color: PGFL_COLOR ): GFL_ERROR; stdcall;

  function gflReduceNoise( var src: GFL_BITMAP; bdst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflDropShadow( var src: GFL_BITMAP; dst: PPGFL_BITMAP;
    size, depth, keep_size: GFL_INT32 ): GFL_ERROR; stdcall;
{$ELSE}  // 2010-03-09 AmigoJack (until ENDIF)
var
  gflGetNumberOfColorsUsed: function( var src: GFL_BITMAP ): GFL_UINT32; stdcall;

  gflNegative: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflBrightness: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; brightness: GFL_INT32 ): GFL_ERROR; stdcall;
  gflContrast: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; contrast: GFL_INT32 ): GFL_ERROR; stdcall;
  gflGamma: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; gamma: double ): GFL_ERROR; stdcall;
  gflLogCorrection: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflNormalize: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflEqualize: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflEqualizeOnLuminance: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflBalance: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; const color: PGFL_COLOR ): GFL_ERROR; stdcall;
  gflAdjust: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; brightness, contrast: GFL_INT32;
    gamma: double ): GFL_ERROR; stdcall;
  gflAdjustHLS: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP;
    hue, lightness, saturation: GFL_INT32 ): GFL_ERROR; stdcall;
  gflAutomaticContrast: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflAutomaticLevels: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;

  gflAverage: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; filter_size: GFL_INT32 ): GFL_ERROR; stdcall;
  gflSoften: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; percentage: GFL_INT32 ): GFL_ERROR; stdcall;
  gflBlur: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; percentage: GFL_INT32 ): GFL_ERROR; stdcall;
  gflGaussianBlur: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; filter_size: GFL_INT32 ): GFL_ERROR; stdcall;
  gflMaximum: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; filter_size: GFL_INT32 ): GFL_ERROR; stdcall;
  gflMinimum: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; filter_size: GFL_INT32 ): GFL_ERROR; stdcall;
  gflMedianBox: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; filter_size: GFL_INT32 ): GFL_ERROR; stdcall;
  gflMedianCross: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; filter_size: GFL_INT32 ): GFL_ERROR; stdcall;
  gflSharpen: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; percentage: GFL_INT32 ): GFL_ERROR; stdcall;

  gflEnhanceDetail: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflEnhanceFocus: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflFocusRestoration: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflEdgeDetectLight: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflEdgeDetectMedium: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflEdgeDetectHeavy: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflEmboss: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflEmbossMore: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;

  gflSepia: function( var src: GFL_BITMAP; bdst: PPGFL_BITMAP; percent: GFL_INT32 ): GFL_ERROR; stdcall;
  gflSepiaEx: function( var src: GFL_BITMAP; bdst: PPGFL_BITMAP; percent: GFL_INT32;
    const color: PGFL_COLOR ): GFL_ERROR; stdcall;

  gflReduceNoise: function( var src: GFL_BITMAP; bdst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflDropShadow: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP;
    size, depth, keep_size: GFL_INT32 ): GFL_ERROR; stdcall;
{$ENDIF}  // 2010-03-09 AmigoJack (from previous IFDEF STATIC ... ELSE)


// CONVOLVE
type
  PGFL_FILTER = ^GFL_FILTER;
  GFL_FILTER = record
    Size: GFL_INT16;
    Matrix: array[{0..15, 0..7} 0..7* 7] of GFL_INT16;  // 2010-03-18 AmigoJack: documentation also says 7
    Divisor,
    Bias: GFL_INT16;
  end;

{$IFDEF STATIC}  // 2010-03-09 AmigoJack
  function gflConvolve( var src: GFL_BITMAP; dst: PPGFL_BITMAP;
    const filter: PGFL_FILTER ): GFL_ERROR; stdcall;
{$ELSE}  // 2010-03-09 AmigoJack (until ENDIF)
var
  gflConvolve: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP;
    const filter: PGFL_FILTER): GFL_ERROR; stdcall;
{$ENDIF}  // 2010-03-09 AmigoJack (from previous IFDEF STATIC ... ELSE)


// SWAP COLORS
const
  GFL_SWAPCOLORS_RBG	= 0;
  GFL_SWAPCOLORS_BGR	= 1;
  GFL_SWAPCOLORS_BRG	= 2;
  GFL_SWAPCOLORS_GRB	= 3;
  GFL_SWAPCOLORS_GBR	= 4;
type
  GFL_SWAPCOLORS_MODE= GFL_UINT16;

{$IFDEF STATIC}  // 2010-03-09 AmigoJack
  function gflSwapColors( var src: GFL_BITMAP; dst: PPGFL_BITMAP; mode: GFL_SWAPCOLORS_MODE ): GFL_ERROR; stdcall;
{$ELSE}  // 2010-03-09 AmigoJack (until ENDIF)
var
  gflSwapColors: function( var src: GFL_BITMAP; dst: PPGFL_BITMAP; mode: GFL_SWAPCOLORS_MODE ): GFL_ERROR; stdcall;
{$ENDIF}  // 2010-03-09 AmigoJack (from previous IFDEF STATIC ... ELSE)


// JPEG LOSSLESS
const
  GFL_LOSSLESS_TRANSFORM_NONE		= 0;
  GFL_LOSSLESS_TRANSFORM_ROTATE90	= 1;
  GFL_LOSSLESS_TRANSFORM_ROTATE180	= 2;
  GFL_LOSSLESS_TRANSFORM_ROTATE270	= 3;
  GFL_LOSSLESS_TRANSFORM_VERTICAL_FLIP	= 4;
  GFL_LOSSLESS_TRANSFORM_HORIZONTAL_FLIP= 5;
type
  GFL_LOSSLESS_TRANSFORM= GFL_UINT16;

{$IFDEF STATIC}  // 2010-03-09 AmigoJack
  function gflJpegLosslessTransform( const filename: PChar; transform: GFL_LOSSLESS_TRANSFORM ): GFL_ERROR; stdcall;
{$ELSE}  // 2010-03-09 AmigoJack (until ENDIF)
var
  gflJpegLosslessTransform: function( const filename: PChar; transform: GFL_LOSSLESS_TRANSFORM ): GFL_ERROR; stdcall;
{$ENDIF}  // 2010-03-09 AmigoJack (from previous IFDEF STATIC ... ELSE)


{$IFDEF MSWINDOWS}

{$IFDEF STATIC}  // 2010-03-09 AmigoJack
  function gflConvertBitmapIntoDIB( const bitmap: PGFL_BITMAP; var hDIB: THandle ): GFL_ERROR; stdcall;
  function gflConvertBitmapIntoDIBSection( const bitmap: PGFL_BITMAP; var hDIB: HBITMAP ): GFL_ERROR; stdcall;
  function gflConvertBitmapIntoDDB( const bitmap: PGFL_BITMAP; var hBitmap: HBITMAP ): GFL_ERROR; stdcall;
  function gflConvertBitmapIntoDDBEx( const bitmap: PGFL_BITMAP; var hBitmap: HBITMAP;
    const background_color: PGFL_COLOR ): GFL_ERROR; stdcall;
  function gflConvertDIBIntoBitmap( hDIB: THandle; bitmapt: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflConvertDDBIntoBitmap( hBitmap: HBITMAP; bitmap: PPGFL_BITMAP ): GFL_ERROR; stdcall;

  function gflLoadBitmapIntoDIB( const filename: PChar; var hDIB: THandle;
    var params: GFL_LOAD_PARAMS; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  function gflLoadBitmapIntoDIBSection( const filename: PChar; var hDib: HBITMAP;
    var params: GFL_LOAD_PARAMS; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  function gflLoadBitmapIntoDDB( const filename: PChar; var hBitmap: HBITMAP;
    var params: GFL_LOAD_PARAMS; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;

  function gflAddText( var bitmap: GFL_BITMAP; const text, font_name: PChar; x, y, font_size,
    orientation: GFL_INT32; italic, bold, strike_out, underline, antialias: GFL_BOOL;
    const color: PGFL_COLOR ): GFL_ERROR; stdcall;
  {$IFDEF UNICODE_SUPPORT}
  function gflAddTextW( var bitmap: GFL_BITMAP; const text, font_name: PWideChar; x, y, font_size,
    orientation: GFL_INT32; italic, bold, strike_out, antialias, underline: GFL_BOOL;
    const color: PGFL_COLOR ): GFL_ERROR; stdcall;
  {$ENDIF}
  function gflGetTextExtent( const text, font_name: PChar; font_size, orientation: GFL_INT32;
    italic, bold, strike_out, underline, antialias: GFL_BOOL;
    var text_width, text_height: GFL_INT32 ): GFL_ERROR; stdcall;

  function gflImportFromClipboard( bitmap: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflExportIntoClipboard( var bitmap: GFL_BITMAP ): GFL_ERROR; stdcall;
  function gflImportFromHWND( hwnd: HWND; const rect: PGFL_RECT; bitmap: PPGFL_BITMAP ): GFL_ERROR; stdcall;
{$ELSE}  // 2010-03-09 AmigoJack (until ENDIF)
var
  gflConvertBitmapIntoDIB: function( const bitmap: PGFL_BITMAP; var hDIB: THandle ): GFL_ERROR; stdcall;
  gflConvertBitmapIntoDIBSection: function( const bitmap: PGFL_BITMAP; var hDIB: HBITMAP ): GFL_ERROR; stdcall;
  gflConvertBitmapIntoDDB: function( const bitmap: PGFL_BITMAP; var hBitmap: HBITMAP ): GFL_ERROR; stdcall;
  gflConvertBitmapIntoDDBEx: function( const src: PGFL_BITMAP; var hBitmap: HBITMAP;
    const background_color: PGFL_COLOR ): GFL_ERROR; stdcall;
  gflConvertDIBIntoBitmap: function( hDIB: THandle; bitmap: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflConvertDDBIntoBitmap: function( hBitmap: HBITMAP; bitmap: PPGFL_BITMAP ): GFL_ERROR; stdcall;

  gflLoadBitmapIntoDIB: function( const filename: PChar; var hDIB: THandle;
    var params: GFL_LOAD_PARAMS; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  gflLoadBitmapIntoDIBSection: function( const filename: PChar; var hDib: HBITMAP;
    var params: GFL_LOAD_PARAMS; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;
  gflLoadBitmapIntoDDB: function( const filename: PChar; var hBitmap: HBITMAP;
    var params: GFL_LOAD_PARAMS; var info: GFL_FILE_INFORMATION ): GFL_ERROR; stdcall;

  gflAddText: function( var bitmap: GFL_BITMAP; const text, font_name: PChar; x, y, font_size,
    orientation: GFL_INT32; italic, bold, strike_out, underline, antialias: GFL_BOOL;
    const color: PGFL_COLOR ): GFL_ERROR; stdcall;
  {$IFDEF UNICODE_SUPPORT}
  gflAddTextW: function( var bitmap: GFL_BITMAP; const text, font_name: PWideChar; x, y, font_size,
    orientation: GFL_INT32; italic, bold, strike_out, antialias, underline: GFL_BOOL;
    const color: PGFL_COLOR ): GFL_ERROR; stdcall;
  {$ENDIF}
  gflGetTextExtent: function( const text, font_name: PChar; font_size, orientation: GFL_INT32;
    italic, bold, strike_out, underline, antialias: GFL_BOOL;
    var text_width, text_height: GFL_INT32 ): GFL_ERROR; stdcall;

  gflImportFromClipboard: function( bitmap: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflExportIntoClipboard: function( var bitmap: GFL_BITMAP ): GFL_ERROR; stdcall;
  gflImportFromHWND: function( hwnd: HWND; const rect: PGFL_RECT; bitmap: PPGFL_BITMAP ): GFL_ERROR; stdcall;
{$ENDIF}  // 2010-03-09 AmigoJack (from previous IFDEF STATIC ... ELSE)


// LINE STYLE
type
  GFL_LINE_STYLE= GFL_UINT32;
const
  GFL_LINE_STYLE_SOLID      = 0;
  GFL_LINE_STYLE_DASH       = 1;
  GFL_LINE_STYLE_DOT        = 2;
  GFL_LINE_STYLE_DASHDOT    = 3;
  GFL_LINE_STYLE_DASHDOTDOT = 4;

{$IFDEF STATIC}  // 2010-03-09 AmigoJack
  function gflDrawPointColor( var src: GFL_BITMAP;
    x, y: GFL_INT32; line_width: GFL_UINT32; const line_color: PGFL_COLOR; bdst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflDrawLineColor( var src: GFL_BITMAP; x0, y0, x1, y1: GFL_INT32; line_width: GFL_UINT32;
    const line_color: PGFL_COLOR; line_style: GFL_LINE_STYLE; bdst: PPGFL_BITMAP): GFL_ERROR; stdcall;
  function gflDrawPolylineColor( var src: GFL_BITMAP; points: array of GFL_POINT;
    num_points: GFL_INT32; line_width: GFL_UINT32; const line_color: PGFL_COLOR;
    line_style: GFL_LINE_STYLE; bdst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflDrawRectangleColor( var src: GFL_BITMAP; x0, y0, width, height: GFL_INT32;
    const fill_color: PGFL_COLOR; line_width: GFL_UINT32; const line_color: PGFL_COLOR;
    line_style: GFL_LINE_STYLE; bdst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflDrawPolygonColor( var src: GFL_BITMAP; points: array of GFL_POINT;
    num_points: GFL_INT32; const fill_color: PGFL_COLOR; line_width: GFL_UINT32;
    const line_color: PGFL_COLOR; line_style: GFL_LINE_STYLE; bdst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  function gflDrawCircleColor( var src: GFL_BITMAP; x, y, radius: GFL_INT32;
    const fill_color: PGFL_COLOR; line_width: GFL_UINT32; const line_color: PGFL_COLOR;
    line_style: GFL_LINE_STYLE; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
{$ELSE}  // 2010-03-09 AmigoJack (until ENDIF)
var
  gflDrawPointColor: function( var src: GFL_BITMAP;
    x, y: GFL_INT32; line_width: GFL_UINT32; const line_color: PGFL_COLOR; bdst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflDrawLineColor: function( var src: GFL_BITMAP; x0, y0, x1, y1: GFL_INT32; line_width: GFL_UINT32;
    const line_color: PGFL_COLOR; line_style: GFL_LINE_STYLE; bdst: PPGFL_BITMAP): GFL_ERROR; stdcall;
  gflDrawPolylineColor: function( var src: GFL_BITMAP; points: array of GFL_POINT;
    num_points: GFL_INT32; line_width: GFL_UINT32; const line_color: PGFL_COLOR;
    line_style: GFL_LINE_STYLE; bdst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflDrawRectangleColor: function( var src: GFL_BITMAP; x0, y0, width, height: GFL_INT32;
    const fill_color: PGFL_COLOR; line_width: GFL_UINT32; const line_color: PGFL_COLOR;
    line_style: GFL_LINE_STYLE; bdst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflDrawPolygonColor: function( var src: GFL_BITMAP; points: array of GFL_POINT;
    num_points: GFL_INT32; const fill_color: PGFL_COLOR; line_width: GFL_UINT32;
    const line_color: PGFL_COLOR; line_style: GFL_LINE_STYLE; bdst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
  gflDrawCircleColor: function( var src: GFL_BITMAP; x, y, radius: GFL_INT32;
    const fill_color: PGFL_COLOR; line_width: GFL_UINT32; const line_color: PGFL_COLOR;
    line_style: GFL_LINE_STYLE; dst: PPGFL_BITMAP ): GFL_ERROR; stdcall;
{$ENDIF}  // 2010-03-09 AmigoJack (from previous IFDEF STATIC ... ELSE)

{$ENDIF}


implementation


// 2010-03-17 AmigoJack: fixed translation of this macro
//   #define gflGetBitmapPtr( _bitmap, _y ) \
//           ((_bitmap)->Data + (_y) * (_bitmap)->BytesPerLine)
function gflGetBitmapPtr( const _bitmap: GFL_Bitmap; const _y: Integer ): Pointer;
begin
  Result:= Pointer( Integer(_bitmap.data)+ (_y* Integer(_bitmap.BytesPerLine)) );
end;


// 2010-03-17 AmigoJack: Macros and Constants for those - are they needed at all?
const
  func_GFL_24BITS	= GFL_RGB or GFL_BGR;
  func_GFL_32BITS	= GFL_RGBA or GFL_ABGR or GFL_BGRA or GFL_ARGB or GFL_CMYK;
  func_GFL_TRUECOLORS	= GFL_24BITS or GFL_32BITS;

{function GFL_IS24BITS( bitmap: PGFL_Bitmap ): Boolean;
begin
  result:= bitmap.BType and func_GFL_24BITS= bitmap.BType;
end;
function GFL_IS32BITS( bitmap: PGFL_Bitmap ): Boolean;
begin
  result:= bitmap.BType and func_GFL_32BITS= bitmap.BType;
end;
function GFL_ISTRUECOLORS( bitmap: PGFL_Bitmap ): Boolean;
begin
  result:= bitmap.BType and func_GFL_TRUECOLORS= bitmap.BType;
end;}
function GFL_ISTRUECOLORS( bitmap: PGFL_Bitmap ): Boolean;
begin
  result:= bitmap.BType and (GFL_RGB or GFL_BGR or GFL_RGBA or GFL_ABGR or GFL_BGRA or GFL_ARGB or GFL_CMYK)= bitmap.BType;
end;


{$IFDEF STATIC}  // 2010-03-09 AmigoJack
function gflMemoryAlloc; external GflDLL;
function gflMemoryRealloc; external GflDLL;
procedure gflMemoryFree; external GflDLL;

function gflGetVersion; external GflDLL;
function gflGetVersionOfLibformat; external GflDLL;

function gflLibraryInit; external GflDLL;
function gflLibraryInitEx; external GflDLL;
procedure gflLibraryExit; external GflDLL;
procedure gflEnableLZW; external GflDLL;
procedure gflSetPluginsPathname; external GflDLL;

function gflGetNumberOfFormat; external GflDLL;
function gflGetFormatIndexByName; external GflDLL;
function gflGetFormatNameByIndex; external GflDLL;
function gflFormatIsSupported; external GflDLL;
function gflFormatIsWritableByIndex; external GflDLL;
function gflFormatIsWritableByName; external GflDLL;
function gflFormatIsReadableByIndex; external GflDLL;
function gflFormatIsReadableByName; external GflDLL;
function gflGetDefaultFormatSuffixByIndex; external GflDLL;
function gflGetDefaultFormatSuffixByName; external GflDLL;
function gflGetFormatDescriptionByIndex; external GflDLL;
function gflGetFormatDescriptionByName; external GflDLL;
function gflGetFormatInformationByIndex; external GflDLL;
function gflGetFormatInformationByName; external GflDLL;

function gflSaveParamsIsSupportedByIndex; external GflDLL;
function gflSaveParamsIsSupportedByName; external GflDLL;
function gflCompressionIsSupportedByIndex; external GflDLL;
function gflCompressionIsSupportedByName; external GflDLL;
function gflBitmapIsSupportedByIndex; external GflDLL;
function gflBitmapIsSupportedByName; external GflDLL;
function gflBitmapTypeIsSupportedByIndex; external GflDLL;
function gflBitmapTypeIsSupportedByName; external GflDLL;

function gflGetErrorString; external GflDLL;
function gflGetLabelForColorModel; external GflDLL;

function gflGetFileInformation; external GflDLL;
function gflGetFileInformationFromHandle; external GflDLL;
function gflGetFileInformationFromMemory; external GflDLL;
procedure gflFreeFileInformation; external GflDLL;

procedure gflGetDefaultLoadParams; external GflDLL;
function gflLoadBitmap; external GflDLL;
function gflLoadBitmapFromHandle; external GflDLL;
function gflLoadBitmapFromMemory; external GflDLL;

procedure gflGetDefaultThumbnailParams; external GflDLL;
function gflLoadThumbnail; external GflDLL;
function gflLoadThumbnailFromHandle; external GflDLL;
function gflLoadThumbnailFromMemory; external GflDLL;

procedure gflGetDefaultSaveParams; external GflDLL;
function gflSaveBitmap; external GflDLL;
function gflSaveBitmapIntoHandle; external GflDLL;
function gflSaveBitmapIntoMemory; external GflDLL;

{$IFDEF UNICODE_SUPPORT}
procedure gflSetPluginsPathnameW; external GflDLL;
function gflGetFileInformationW; external GflDLL;
function gflLoadBitmapW; external GflDLL;
function gflLoadThumbnailW; external GflDLL;
function gflSaveBitmapW; external GflDLL;
{$ENDIF}

function gflFileCreate; external GflDLL;
function gflFileAddPicture; external GflDLL;
procedure gflFileClose; external GflDLL;

{$IFDEF UNICODE_SUPPORT}
function gflFileCreateW; external GflDLL;
{$ENDIF}

function gflAllockBitmap; external GflDLL;
function gflAllockBitmapEx; external GflDLL;
procedure gflFreeBitmap; external GflDLL;
procedure gflFreeBitmapData; external GflDLL;
function gflCloneBitmap; external GflDLL;
procedure gflBitmapSetName; external GflDLL;

function gflGetExtraInfosCount; external GflDLL;
procedure gflGetExtraInfos; external GflDLL;

function gflBitmapHasEXIF; external GflDLL;
function gflBitmapHasIPTC; external GflDLL;
function gflBitmapHasICCProfile; external GflDLL;

procedure gflBitmapRemoveEXIFThumbnail; external GflDLL;
procedure gflBitmapRemoveICCProfile; external GflDLL;
procedure gflBitmapGetICCProfile; external GflDLL;
procedure gflBitmapCopyICCProfile; external GflDLL;
procedure gflBitmapRemoveMetaData; external GflDLL;

function gflBitmapGetXMP; external GflDLL;

function gflLoadEXIF; external GflDLL;

function gflBitmapGetIPTC; external GflDLL;
function gflBitmapGetIPTCValue; external GflDLL;
function gflNewIPTC; external GflDLL;
procedure gflFreeIPTC; external GflDLL;
function gflSetIPTCValue; external GflDLL;
function gflRemoveIPTCValue; external GflDLL;

function gflLoadIPTC; external GflDLL;
function gflSaveIPTC; external GflDLL;
function gflBitmapSetIPTC; external GflDLL;

{$IFDEF UNICODE_SUPPORT}
function gflLoadEXIFW; external GflDLL;
function gflLoadIPTCW; external GflDLL;
function gflSaveIPTCW; external GflDLL;
{$ENDIF}

function gflBitmapGetEXIF; external GflDLL;
function gflBitmapGetEXIFValue; external GflDLL;
procedure gflFreeEXIF; external GflDLL;
procedure gflBitmapSetComment; external GflDLL;

function gflBitmapGetEXIF2; external GflDLL;
procedure gflFreeEXIF2; external GflDLL;
procedure gflBitmapSetEXIF2; external GflDLL;
procedure gflBitmapSetEXIFValueString2; external GflDLL;
procedure gflBitmapSetEXIFValueInt2; external GflDLL;
procedure gflBitmapSetEXIFValueRational2; external GflDLL;
procedure gflBitmapSetEXIFValueRationalArray2; external GflDLL;

function gflJPEGGetComment; external GflDLL;
function gflJPEGSetComment; external GflDLL;
function gflPNGGetComment; external GflDLL;
function gflPNGSetComment; external GflDLL;

{$IFDEF UNICODE_SUPPORT}
function gflJPEGGetCommentW; external GflDLL;
function gflJPEGSetCommentW; external GflDLL;
function gflPNGGetCommentW; external GflDLL;
function gflPNGSetCommentW; external GflDLL;
{$ENDIF}

function gflIsLutFile; external GflDLL;
function gflIsCompatibleLutFile; external GflDLL;
function gflApplyLutFile; external GflDLL;

function gflResize; external GflDLL;

function gflChangeColorDepth; external GflDLL;

function gflGetColorAt; external GflDLL;
function gflSetColorAt; external GflDLL;
function gflGetColorAtEx; external GflDLL;
function gflSetColorAtEx; external GflDLL;

function gflFlipVertical; external GflDLL;
function gflFlipHorizontal; external GflDLL;
function gflCrop; external GflDLL;
function gflAutoCrop; external GflDLL;
function gflAutoCrop2; external GflDLL;

function gflResizeCanvas; external GflDLL;
function gflScaleToGrey; external GflDLL;

function gflRotate; external GflDLL;
function gflRotateFine; external GflDLL;
function gflReplaceColor; external GflDLL;

function gflBitblt; external GflDLL;
function gflBitbltEx; external GflDLL;
function gflMerge; external GflDLL;
function gflCombineAlpha; external GflDLL;
function gflSetTransparentColor; external GflDLL;


// functions in LibGflExxx.dll
function gflGetNumberOfColorsUsed; external GfleDLL;

function gflNegative; external GfleDLL;
function gflBrightness; external GfleDLL;
function gflContrast; external GfleDLL;
function gflGamma; external GfleDLL;
function gflLogCorrection; external GfleDLL;
function gflNormalize; external GfleDLL;
function gflEqualize; external GfleDLL;
function gflEqualizeOnLuminance; external GfleDLL;
function gflBalance; external GfleDLL;
function gflAdjust; external GfleDLL;
function gflAdjustHLS; external GfleDLL;
function gflAutomaticContrast; external GfleDLL;
function gflAutomaticLevels; external GfleDLL;

function gflAverage; external GfleDLL;
function gflSoften; external GfleDLL;
function gflBlur; external GfleDLL;
function gflGaussianBlur; external GfleDLL;
function gflMaximum; external GfleDLL;
function gflMinimum; external GfleDLL;
function gflMedianBox; external GfleDLL;
function gflMedianCross; external GfleDLL;
function gflSharpen; external GfleDLL;

function gflEnhanceDetail; external GfleDLL;
function gflEnhanceFocus; external GfleDLL;
function gflFocusRestoration; external GfleDLL;
function gflEdgeDetectLight; external GfleDLL;
function gflEdgeDetectMedium; external GfleDLL;
function gflEdgeDetectHeavy; external GfleDLL;
function gflEmboss; external GfleDLL;
function gflEmbossMore; external GfleDLL;

function gflSepia; external GfleDLL;
function gflSepiaEx; external GfleDLL;

function gflReduceNoise; external GfleDLL;
function gflDropShadow; external GfleDLL;

function gflConvolve; external GfleDLL;

function gflSwapColors; external GfleDLL;

function gflJpegLosslessTransform; external GfleDLL;

{$IFDEF MSWINDOWS}
function gflConvertBitmapIntoDIB; external GfleDLL;
function gflConvertBitmapIntoDIBSection; external GfleDLL;
function gflConvertBitmapIntoDDB; external GfleDLL;
function gflConvertBitmapIntoDDBEx; external GfleDLL;
function gflConvertDIBIntoBitmap; external GfleDLL;
function gflConvertDDBIntoBitmap; external GfleDLL;

function gflLoadBitmapIntoDIB; external GfleDLL;
function gflLoadBitmapIntoDIBSection; external GfleDLL;
function gflLoadBitmapIntoDDB; external GfleDLL;

function gflAddText; external GfleDLL;
{$IFDEF UNICODE_SUPPORT}
function gflAddTextW; external GfleDLL;
{$ENDIF}
function gflGetTextExtent; external GfleDLL;

function gflImportFromClipboard; external GfleDLL;
function gflExportIntoClipboard; external GfleDLL;
function gflImportFromHWND; external GfleDLL;

function gflDrawPointColor; external GfleDLL;
function gflDrawLineColor; external GfleDLL;
function gflDrawPolylineColor; external GfleDLL;
function gflDrawRectangleColor; external GfleDLL;
function gflDrawPolygonColor; external GfleDLL;
function gflDrawCircleColor; external GfleDLL;
{$ENDIF}

{$ELSE}  // 2010-03-09 AmigoJack: above was static, below is dynamic (until ENDIF)

// 2010-03-09 AmigoJack:
//   For internal purposes only: converts a PChar into a String.
//   Redefined to not fully include SysUtils only for this one function.
function _StrPas( const Str: PChar ): String;
begin
  result:= Str;
end;

// 2010-03-09 AmigoJack:
//   For internal purposes only: converts a PWideChar into a WideString.
{$IFDEF UNICODE_SUPPORT}
function _StrPasW( const WStr: PWideChar ): WideString;
begin
  result:= WStr;
end;
{$ENDIF}

// 2010-03-09 AmigoJack:
//   For internal purposes only: holds all characters for a textual representation of
//   an ordinal (a number) up to base 16 (which includes decimal and hexadecimal).
const
  _SKY_NUMBERS= '0123456789ABCDEF';

// 2010-03-09 AmigoJack:
//   For internal purposes only: converts an ordinal (unsigned integer) to text
//   including various options.
function _IntToSky
( iValue: DWord  // Ordinal to convert - we only need 32bit unsigned
; iAmp: Byte= 10  // Base; defaults to 10 (decimal). Other well-known formats could be:
                  // - 2 = binary
                  // - 8 = octal
                  // - 16 = hexadecimal (highest possible value)
; iStock: Byte= 0  // Minimal length of resulting text. If the result is shorter then
                   // leading zeroes will be added.
): String;  // Ordinal as text
begin
  result:= '';
  repeat  // For each digit of the ordinal...
    result:= _SKY_NUMBERS[(iValue mod iAmp)+ 1]+ result;  // ...get appropriate character representation
    iValue:= iValue div iAmp;  // Decrement input by what we converted so far
  until iValue= 0;

  while Length( result )< iStock do result:= _SKY_NUMBERS[1]+ result;  // Fill with zeroes in front
end;

// 2010-03-09 AmigoJack:
//   For internal purposes only: gets a textual message for the last API-error-number.
//   Unicode version.
{$IFDEF UNICODE_SUPPORT}
function _GetLastErrorMsg
( pErr: PDWord= nil  // Number upon which to get the text. If not given, GetLastError will be called
): WideString;  // Textual message
var
  iErr: DWord;  // Actual error-number to be used
  w1: PWideChar;  // Buffer for message
begin
  if pErr= nil then iErr:= GetLastError else iErr:= pErr^;  // No error-number given will call GetLastError

  GetMem( w1, 20000 );  // Reserve space for 10000 characters
  FormatMessageW( FORMAT_MESSAGE_IGNORE_INSERTS or FORMAT_MESSAGE_FROM_SYSTEM, nil, iErr, 0, w1, 10000, nil );  // Get message
  result:= _StrPasW( w1 );  // Convert to WideString
  FreeMem( w1 );  // Release space
  if Length( result )> 0 then while result[Length( result )] in [WideChar(#13), WideChar(#10)] do Delete( result, Length( result ), 1 );  // Kill trailing linebreaks
  result:= _IntToSky( iErr )+ ' (0x'+ _IntToSky( iErr, 16, 8 )+ ') '+ result;  // Append error-number to message itself, once in decimal, once in hexadecimal representation
end;
{$ELSE}
function _GetLastErrorMsg( pErr: PDWord= nil ): String;  // Same as above, but Ansi version
var
  iErr: DWord;
  p1: PChar;  // Buffer for message (only Ansi)
begin
  if pErr= nil then iErr:= GetLastError else iErr:= pErr^;

  GetMem( p1, 10000 );  // Reserve space for 10000 characters
  FormatMessage( FORMAT_MESSAGE_IGNORE_INSERTS or FORMAT_MESSAGE_FROM_SYSTEM, nil, iErr, 0, p1, 10000, nil );
  result:= _StrPas( p1 );  // Convert to String
  freemem( p1 );
  if Length( result )> 0 then while result[Length( result )] in [#13, #10] do Delete( result, Length( result ), 1 );
  result:= _IntToSky( iErr )+ ' (0x'+ _IntToSky( iErr, 16, 8 )+ ') '+ result;
end;
{$ENDIF}

// 2010-03-09 AmigoJack:
//   For internal purposes only: link a function/procedure to the DLL.
function _gfl_GetProcAddress
{$IFDEF UNICODE_SUPPORT}
( sName: PChar  // Name of function/procedure to search for in the DLL
; var pAddr: Pointer  // Variable receiving the link (if found)
; hLib: THandle  // DLL to link to
; var sFail: WideString  // Text to give back possible errors (adds a line)
{$ELSE}  // Same as above, only Ansi version
( sName: PChar
; var pAddr: Pointer
; hLib: THandle
; var sFail: String  // Text to give back possible errors
{$ENDIF}
): boolean;  // Did we succeed? Otherwise we should lookup "sFail" for errors
var
  iErr: DWord;  // Will hold error-number which would otherwise be overwritten too early by others
begin
  pAddr:= GetProcAddress( hLib, sName );  // Get link
  result:= pAddr<> nil;  // If link is NIL the function failed
  if not result then begin  // On fail catch the error
    iErr:= GetLastError;  // Get error-number

    sFail:= sFail+ #13#10+  // Append error-pointer-text with linebreak,...
    'libgfl.GetProc: "'+ _StrPas( sName )+  // ...a source where an error occured and...
    '" failed: '+ _GetLastErrorMsg( @iErr );  // ...the actual error (most probably #127 "procedure not found")
  end;
end;

// 2010-03-09 AmigoJack:
//   For internal purposes only: remembers if we called GflLibraryInit
var
  _bGflLibraryInit: boolean;

// 2010-03-09 AmigoJack:
//   Link, load and initialize LibGfl
function gfl_LoadLib
{$IFDEF UNICODE_SUPPORT}
( sPathLibGfl  // Path + filename for base DLL (i.e. "libgfl311.dll")
, sPathLibGfle  // Path + filename for enhancement/extra DLL (i.e. "libgfle311.dll")
, sPathPlugins: WideString  // Path to plugins; gflSetPluginsPathname won't be called if empty
; var sFail: WideString  // Gives back possible errors
{$ELSE}  // Same as above, only Ansi version
( sPathLibGfl
, sPathLibGfle,
, sPathPlugins: String
; var sFail: String
{$ENDIF}
): boolean;  // Did we succeed? Otherwise see "sFail"
const
  INITFAIL= #13#10'libgfl.Load Init: failed: ';
var
  bGfl,  // Is base DLL loaded?
  bGfle: boolean;  // Is enhancement/extra DLL loaded?
  iFail: GFL_ERROR;  // Return of gflLibraryInit
begin
{$IFDEF UNICODE_SUPPORT}
  if hLibGfl= 0 then begin  // Base DLL yet not loaded?
    hLibGfl:= LoadLibraryW( PWideChar( sPathLibGfl ) );  // Load it

    if hLibGfl= 0 then sFail:= sFail+ #13#10+  // Handle 0 means an error occured - append to text...
    'libgfl.Load gfl: "'+ sPathLibGfl+  // ...where the error occured and...
    '" failed: '+ _GetLastErrorMsg;  // ...the actual error (most probably #126 "module not found")
  end;

  if hLibGfle= 0 then begin  // Enhancement/extra DLL yet not loaded?
    hLibGfle:= LoadLibraryW( PWideChar( sPathLibGfle ) );  // Load it

    if hLibGfle= 0 then sFail:= sFail+ #13#10+  // Handle 0 means an error occured. See above
    'libgfl.Load gfle: "'+ sPathLibGfle+
    '" failed: '+ _GetLastErrorMsg;
  end;
{$ELSE}  // Same as above, Ansi version
  if hLibGfl= 0 then begin
    hLibGfl:= LoadLibrary( PChar( sPathLibGfl ) );

    if hLibGfl= 0 then sFail:= sFail+ #13#10+
    'libgfl.Load gfl: "'+ sPathLibGfl+
    '" failed: '+ _GetLastErrorMsg;
  end;

  if hLibGfle= 0 then begin
    hLibGfle:= LoadLibrary( PChar( sPathLibGfle ) );

    if hLibGfle= 0 then sFail:= sFail+ #13#10+
    'libgfl.Load gfle: "'+ sPathLibGfle+
    '" failed: '+ _GetLastErrorMsg;
  end;
{$ENDIF}

  bGfl:= false;  // Base DLL load yet unsatisfied
  if hLibGfl<> 0 then begin  // DLL loaded? Try to link each function/procedure
    _gfl_GetProcAddress( 'gflMemoryAlloc', @gflMemoryAlloc, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflMemoryRealloc', @gflMemoryRealloc, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflMemoryFree', @gflMemoryFree, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflGetVersion', @gflGetVersion, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflGetVersionOfLibformat', @gflGetVersionOfLibformat, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflLibraryInit', @gflLibraryInit, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflLibraryInitEx', @gflLibraryInitEx, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflLibraryExit', @gflLibraryExit, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflEnableLZW', @gflEnableLZW, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflSetPluginsPathname', @gflSetPluginsPathname, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflGetNumberOfFormat', @gflGetNumberOfFormat, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflGetFormatIndexByName', @gflGetFormatIndexByName, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflGetFormatNameByIndex', @gflGetFormatNameByIndex, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflFormatIsSupported', @gflFormatIsSupported, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflFormatIsWritableByIndex', @gflFormatIsWritableByIndex, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflFormatIsWritableByName', @gflFormatIsWritableByName, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflFormatIsReadableByIndex', @gflFormatIsReadableByIndex, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflFormatIsReadableByName', @gflFormatIsReadableByName, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflGetDefaultFormatSuffixByIndex', @gflGetDefaultFormatSuffixByIndex, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflGetDefaultFormatSuffixByName', @gflGetDefaultFormatSuffixByName, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflGetFormatDescriptionByIndex', @gflGetFormatDescriptionByIndex, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflGetFormatDescriptionByName', @gflGetFormatDescriptionByName, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflGetFormatInformationByIndex', @gflGetFormatInformationByIndex, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflGetFormatInformationByName', @gflGetFormatInformationByName, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflSaveParamsIsSupportedByIndex', @gflSaveParamsIsSupportedByIndex, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflSaveParamsIsSupportedByName', @gflSaveParamsIsSupportedByName, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflCompressionIsSupportedByIndex', @gflCompressionIsSupportedByIndex, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflCompressionIsSupportedByName', @gflCompressionIsSupportedByName, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflBitmapIsSupportedByIndex', @gflBitmapIsSupportedByIndex, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflBitmapIsSupportedByName', @gflBitmapIsSupportedByName, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflBitmapTypeIsSupportedByIndex', @gflBitmapTypeIsSupportedByIndex, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflBitmapTypeIsSupportedByName', @gflBitmapTypeIsSupportedByName, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflGetErrorString', @gflGetErrorString, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflGetLabelForColorModel', @gflGetLabelForColorModel, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflGetFileInformation', @gflGetFileInformation, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflGetFileInformationFromHandle', @gflGetFileInformationFromHandle, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflGetFileInformationFromMemory', @gflGetFileInformationFromMemory, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflFreeFileInformation', @gflFreeFileInformation, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflGetDefaultLoadParams', @gflGetDefaultLoadParams, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflLoadBitmap', @gflLoadBitmap, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflLoadBitmapFromHandle', @gflLoadBitmapFromHandle, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflLoadBitmapFromMemory', @gflLoadBitmapFromMemory, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflGetDefaultThumbnailParams', @gflGetDefaultThumbnailParams, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflLoadThumbnail', @gflLoadThumbnail, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflLoadThumbnailFromHandle', @gflLoadThumbnailFromHandle, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflLoadThumbnailFromMemory', @gflLoadThumbnailFromMemory, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflGetDefaultSaveParams', @gflGetDefaultSaveParams, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflSaveBitmap', @gflSaveBitmap, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflSaveBitmapIntoHandle', @gflSaveBitmapIntoHandle, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflSaveBitmapIntoMemory', @gflSaveBitmapIntoMemory, hLibGfl, sFail );

    {$IFDEF UNICODE_SUPPORT}
    _gfl_GetProcAddress( 'gflSetPluginsPathnameW', @gflSetPluginsPathnameW, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflGetFileInformationW', @gflGetFileInformationW, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflLoadBitmapW', @gflLoadBitmapW, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflLoadThumbnailW', @gflLoadThumbnailW, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflSaveBitmapW', @gflSaveBitmapW, hLibGfl, sFail );
    {$ENDIF}

    _gfl_GetProcAddress( 'gflFileCreate', @gflFileCreate, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflFileAddPicture', @gflFileAddPicture, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflFileClose', @gflFileClose, hLibGfl, sFail );

    {$IFDEF UNICODE_SUPPORT}
    _gfl_GetProcAddress( 'gflFileCreateW', @gflFileCreateW, hLibGfl, sFail );
    {$ENDIF}

    _gfl_GetProcAddress( 'gflAllockBitmap', @gflAllockBitmap, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflAllockBitmapEx', @gflAllockBitmapEx, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflFreeBitmap', @gflFreeBitmap, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflFreeBitmapData', @gflFreeBitmapData, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflCloneBitmap', @gflCloneBitmap, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflBitmapSetName', @gflBitmapSetName, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflGetExtraInfosCount', @gflGetExtraInfosCount, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflGetExtraInfos', @gflGetExtraInfos, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflBitmapHasEXIF', @gflBitmapHasEXIF, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflBitmapHasIPTC', @gflBitmapHasIPTC, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflBitmapHasICCProfile', @gflBitmapHasICCProfile, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflBitmapRemoveEXIFThumbnail', @gflBitmapRemoveEXIFThumbnail, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflBitmapRemoveICCProfile', @gflBitmapRemoveICCProfile, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflBitmapGetICCProfile', @gflBitmapGetICCProfile, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflBitmapCopyICCProfile', @gflBitmapCopyICCProfile, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflBitmapRemoveMetaData', @gflBitmapRemoveMetaData, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflBitmapGetXMP', @gflBitmapGetXMP, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflLoadEXIF', @gflLoadEXIF, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflBitmapGetIPTC', @gflBitmapGetIPTC, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflBitmapGetIPTCValue', @gflBitmapGetIPTCValue, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflNewIPTC', @gflNewIPTC, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflFreeIPTC', @gflFreeIPTC, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflSetIPTCValue', @gflSetIPTCValue, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflRemoveIPTCValue', @gflRemoveIPTCValue, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflLoadIPTC', @gflLoadIPTC, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflSaveIPTC', @gflSaveIPTC, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflBitmapSetIPTC', @gflBitmapSetIPTC, hLibGfl, sFail );

    {$IFDEF UNICODE_SUPPORT}
    _gfl_GetProcAddress( 'gflLoadEXIFW', @gflLoadEXIFW, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflLoadIPTCW', @gflLoadIPTCW, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflSaveIPTCW', @gflSaveIPTCW, hLibGfl, sFail );
    {$ENDIF}

    _gfl_GetProcAddress( 'gflBitmapGetEXIF', @gflBitmapGetEXIF, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflBitmapGetEXIFValue', @gflBitmapGetEXIFValue, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflFreeEXIF', @gflFreeEXIF, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflBitmapSetComment', @gflBitmapSetComment, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflBitmapGetEXIF2', @gflBitmapGetEXIF2, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflFreeEXIF2', @gflFreeEXIF2, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflBitmapSetEXIF2', @gflBitmapSetEXIF2, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflBitmapSetEXIFValueString2', @gflBitmapSetEXIFValueString2, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflBitmapSetEXIFValueInt2', @gflBitmapSetEXIFValueInt2, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflBitmapSetEXIFValueRational2', @gflBitmapSetEXIFValueRational2, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflBitmapSetEXIFValueRationalArray2', @gflBitmapSetEXIFValueRationalArray2, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflJPEGGetComment', @gflJPEGGetComment, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflJPEGSetComment', @gflJPEGSetComment, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflPNGGetComment', @gflPNGGetComment, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflPNGSetComment', @gflPNGSetComment, hLibGfl, sFail );

    {$IFDEF UNICODE_SUPPORT}
    _gfl_GetProcAddress( 'gflJPEGGetCommentW', @gflJPEGGetCommentW, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflJPEGSetCommentW', @gflJPEGSetCommentW, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflPNGGetCommentW', @gflPNGGetCommentW, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflPNGSetCommentW', @gflPNGSetCommentW, hLibGfl, sFail );
    {$ENDIF}

    _gfl_GetProcAddress( 'gflIsLutFile', @gflIsLutFile, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflIsCompatibleLutFile', @gflIsCompatibleLutFile, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflApplyLutFile', @gflApplyLutFile, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflResize', @gflResize, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflChangeColorDepth', @gflChangeColorDepth, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflGetColorAt', @gflGetColorAt, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflSetColorAt', @gflSetColorAt, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflGetColorAtEx', @gflGetColorAtEx, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflSetColorAtEx', @gflSetColorAtEx, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflFlipVertical', @gflFlipVertical, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflFlipHorizontal', @gflFlipHorizontal, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflCrop', @gflCrop, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflAutoCrop', @gflAutoCrop, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflAutoCrop2', @gflAutoCrop2, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflResizeCanvas', @gflResizeCanvas, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflScaleToGrey', @gflScaleToGrey, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflRotate', @gflRotate, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflRotateFine', @gflRotateFine, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflReplaceColor', @gflReplaceColor, hLibGfl, sFail );

    _gfl_GetProcAddress( 'gflBitblt', @gflBitblt, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflBitbltEx', @gflBitbltEx, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflMerge', @gflMerge, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflCombineAlpha', @gflCombineAlpha, hLibGfl, sFail );
    _gfl_GetProcAddress( 'gflSetTransparentColor', @gflSetTransparentColor, hLibGfl, sFail );


    if Length( sPathPlugins )> 0 then begin  // Set Plugin path?
      // Only try if link to DLL has been established (Assigned)

      {$IFDEF UNICODE_SUPPORT}
      if Assigned( gflSetPluginsPathnameW ) then gflSetPluginsPathnameW( PWideChar( sPathPlugins ) );
      {$ELSE}
      if Assigned( gflSetPluginsPathname ) then gflSetPluginsPathname( PChar( sPathPlugins ) );
      {$ENDIF}
    end;

    bGfl:= true;  // Now we are satisfied: DLL is loaded and no exception occured until now
  end;

  // functions in LibGflExxx.dll
  bGfle:= false;  // Enhancement/extra DLL load yet unsatisfied
  if hLibGfle<> 0 then begin  // DLL loaded? Try to link each function/procedure
    _gfl_GetProcAddress( 'gflGetNumberOfColorsUsed', @gflGetNumberOfColorsUsed, hLibGfle, sFail );

    _gfl_GetProcAddress( 'gflNegative', @gflNegative, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflBrightness', @gflBrightness, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflContrast', @gflContrast, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflGamma', @gflGamma, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflLogCorrection', @gflLogCorrection, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflNormalize', @gflNormalize, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflEqualize', @gflEqualize, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflEqualizeOnLuminance', @gflEqualizeOnLuminance, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflBalance', @gflBalance, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflAdjust', @gflAdjust, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflAdjustHLS', @gflAdjustHLS, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflAutomaticContrast', @gflAutomaticContrast, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflAutomaticLevels', @gflAutomaticLevels, hLibGfle, sFail );

    _gfl_GetProcAddress( 'gflAverage', @gflAverage, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflSoften', @gflSoften, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflBlur', @gflBlur, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflGaussianBlur', @gflGaussianBlur, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflMaximum', @gflMaximum, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflMinimum', @gflMinimum, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflMedianBox', @gflMedianBox, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflMedianCross', @gflMedianCross, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflSharpen', @gflSharpen, hLibGfle, sFail );

    _gfl_GetProcAddress( 'gflEnhanceDetail', @gflEnhanceDetail, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflEnhanceFocus', @gflEnhanceFocus, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflFocusRestoration', @gflFocusRestoration, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflEdgeDetectLight', @gflEdgeDetectLight, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflEdgeDetectMedium', @gflEdgeDetectMedium, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflEdgeDetectHeavy', @gflEdgeDetectHeavy, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflEmboss', @gflEmboss, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflEmbossMore', @gflEmbossMore, hLibGfle, sFail );

    _gfl_GetProcAddress( 'gflSepia', @gflSepia, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflSepiaEx', @gflSepiaEx, hLibGfle, sFail );

    _gfl_GetProcAddress( 'gflReduceNoise', @gflReduceNoise, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflDropShadow', @gflDropShadow, hLibGfle, sFail );

    _gfl_GetProcAddress( 'gflConvolve', @gflConvolve, hLibGfle, sFail );

    _gfl_GetProcAddress( 'gflSwapColors', @gflSwapColors, hLibGfle, sFail );

    _gfl_GetProcAddress( 'gflJpegLosslessTransform', @gflJpegLosslessTransform, hLibGfle, sFail );

    {$IFDEF MSWINDOWS}
    _gfl_GetProcAddress( 'gflConvertBitmapIntoDIB', @gflConvertBitmapIntoDIB, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflConvertBitmapIntoDIBSection', @gflConvertBitmapIntoDIBSection, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflConvertBitmapIntoDDB', @gflConvertBitmapIntoDDB, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflConvertBitmapIntoDDBEx', @gflConvertBitmapIntoDDBEx, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflConvertDIBIntoBitmap', @gflConvertDIBIntoBitmap, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflConvertDDBIntoBitmap', @gflConvertDDBIntoBitmap, hLibGfle, sFail );

    _gfl_GetProcAddress( 'gflLoadBitmapIntoDIB', @gflLoadBitmapIntoDIB, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflLoadBitmapIntoDIBSection', @gflLoadBitmapIntoDIBSection, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflLoadBitmapIntoDDB', @gflLoadBitmapIntoDDB, hLibGfle, sFail );

    _gfl_GetProcAddress( 'gflAddText', @gflAddText, hLibGfle, sFail );
    {$IFDEF UNICODE_SUPPORT}
    _gfl_GetProcAddress( 'gflAddTextW', @gflAddTextW, hLibGfle, sFail );
    {$ENDIF}
    _gfl_GetProcAddress( 'gflGetTextExtent', @gflGetTextExtent, hLibGfle, sFail );

    _gfl_GetProcAddress( 'gflImportFromClipboard', @gflImportFromClipboard, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflExportIntoClipboard', @gflExportIntoClipboard, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflImportFromHWND', @gflImportFromHWND, hLibGfle, sFail );

    _gfl_GetProcAddress( 'gflDrawPointColor', @gflDrawPointColor, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflDrawLineColor', @gflDrawLineColor, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflDrawPolylineColor', @gflDrawPolylineColor, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflDrawRectangleColor', @gflDrawRectangleColor, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflDrawPolygonColor', @gflDrawPolygonColor, hLibGfle, sFail );
    _gfl_GetProcAddress( 'gflDrawCircleColor', @gflDrawCircleColor, hLibGfle, sFail );
    {$ENDIF}


    bGfle:= true;  // Now we are satisfied: DLL is loaded and no exception occured until now
  end;

  result:= bGfl and bGfle;  // Both DLLs are at least loaded
  if result and (not _bGflLibraryInit) then begin  // Succeeded and yet never called gflLibraryInit?
    iFail:= gflLibraryInit;
    case iFail of
      GFL_NO_ERROR: begin
        _bGflLibraryInit:= true;  // Only call gflLibraryInit once (successful), so remember it
        exit;
      end;

      // We cannot expect gflGetErrorString() to be working here
      GFL_ERROR_FILE_OPEN: sFail:= sFail+ INITFAIL+ 'File open';
      GFL_ERROR_FILE_READ: sFail:= sFail+ INITFAIL+ 'File read';
      GFL_ERROR_FILE_CREATE: sFail:= sFail+ INITFAIL+ 'File create';
      GFL_ERROR_FILE_WRITE: sFail:= sFail+ INITFAIL+ 'File write';
      GFL_ERROR_NO_MEMORY: sFail:= sFail+ INITFAIL+ 'No memory';
      GFL_ERROR_UNKNOWN_FORMAT: sFail:= sFail+ INITFAIL+ 'Unknown format';
      GFL_ERROR_BAD_BITMAP: sFail:= sFail+ INITFAIL+ 'Bad bitmap';
      GFL_ERROR_BAD_FORMAT_INDEX: sFail:= sFail+ INITFAIL+ 'Bad format index';
      GFL_ERROR_BAD_PARAMETERS: sFail:= sFail+ INITFAIL+ 'Bad parameters';
      GFL_UNKNOWN_ERROR: sFail:= sFail+ INITFAIL+ 'Unknown error';
      else sFail:= sFail+ INITFAIL+ '#'+ _IntToSky( iFail );
    end;

    result:= false;  // Coming here means something failed
  end;
end;

// 2010-03-09 AmigoJack:
//   Exit and unload LibGfl
function gfl_UnloadLib
{$IFDEF UNICODE_SUPPORT}
( var sFail: WideString  // Gives back possible errors
{$ELSE}  // Same as above, only Ansi version
( var sFail: String
{$ENDIF}
): boolean;  // Did we succeed? Otherwise see "sFail"
begin
  result:= false;
  try
    if Assigned( gflLibraryExit ) then gflLibraryExit;

    if hLibGfl<> 0 then begin  // Base DLL yet loaded?
      if not FreeLibrary( hLibGfl ) then sFail:= sFail+ #13#10+  // an error occured - append to text...
      'libgfl.Unload gfl: '+  // ...where the error occured and...
      ' failed: '+ _GetLastErrorMsg;  // ...the actual error

      hLibGfl:= 0;
    end;

    if hLibGfle<> 0 then begin  // Enhancement/extra DLL yet loaded?
      if not FreeLibrary( hLibGfle ) then sFail:= sFail+ #13#10+  // an error occured. See above
      'libgfl.Unload gfle: '+
      ' failed: '+ _GetLastErrorMsg;

      hLibGfle:= 0;
    end;

    _bGflLibraryInit:= false;  // Reset calling gflLibraryInit
    result:= true;
  except
    sFail:= sFail+ #13#10+
    'libgfl.Unload exception';
  end;
end;
{$ENDIF}  // 2010-03-09 AmigoJack (from previous IFDEF STATIC ... ELSE)

//******************************************************************************

{$IFDEF STATIC}  // 2010-03-09 AmigoJack
initialization
  gflLibraryInit;
finalization
  gflLibraryExit;
{$ELSE}  // 2010-03-09 AmigoJack: dynamic linking (until ENDIF)
initialization
  _bGflLibraryInit:= false;  // not initialized yet
finalization
  if _bGflLibraryInit then gflLibraryExit;  // only call if we actually called gflLibraryInit
{$ENDIF}  // 2010-03-09 AmigoJack: end of dynamic linking

end.

